# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import
import copy
import datetime
import logging
import os

import dateutil.parser as dateutil_parser

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.mixins import SaltReturnAssertsMixin

# Import Salt Testing Libs
from tests.support.mock import MagicMock, patch
import tests.integration as integration

# Import Salt libs
import salt.utils.schedule

from salt.modules.test import ping as ping

log = logging.getLogger(__name__)
ROOT_DIR = os.path.join(integration.TMP, 'schedule-unit-tests')
SOCK_DIR = os.path.join(ROOT_DIR, 'test-socks')

DEFAULT_CONFIG = salt.config.minion_config(None)
DEFAULT_CONFIG['conf_dir'] = ROOT_DIR
DEFAULT_CONFIG['root_dir'] = ROOT_DIR
DEFAULT_CONFIG['sock_dir'] = SOCK_DIR
DEFAULT_CONFIG['pki_dir'] = os.path.join(ROOT_DIR, 'pki')
DEFAULT_CONFIG['cachedir'] = os.path.join(ROOT_DIR, 'cache')


class SchedulerPostponeTest(ModuleCase, SaltReturnAssertsMixin):
    '''
    Validate the pkg module
    '''
    def setUp(self):
        with patch('salt.utils.schedule.clean_proc_dir', MagicMock(return_value=None)):
            functions = {'test.ping': ping}
            self.schedule = salt.utils.schedule.Schedule(copy.deepcopy(DEFAULT_CONFIG), functions, returners={})
        self.schedule.opts['loop_interval'] = 1

    def tearDown(self):
        self.schedule.reset()

    def test_postpone(self):
        '''
        verify that scheduled job is postponed until the specified time.
        '''
        job = {
          'schedule': {
            'job1': {
              'function': 'test.ping',
              'when': '11/29/2017 4pm',
            }
          }
        }

        # 11/29/2017 4pm
        run_time = dateutil_parser.parse('11/29/2017 4:00pm')

        # 5 minute delay
        delay = 300

        # Add job to schedule
        self.schedule.opts.update(job)

        # Postpone the job by 5 minutes
        self.schedule.postpone_job('job1', {'time': run_time.strftime('%Y-%m-%dT%H:%M:%S'),
                                            'new_time': (run_time + datetime.timedelta(seconds=delay)).strftime('%Y-%m-%dT%H:%M:%S')})

        # Run at the original time
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status('job1')
        self.assertNotIn('_last_run', ret)

        # Run 5 minutes later
        self.schedule.eval(now=run_time + datetime.timedelta(seconds=delay))
        ret = self.schedule.job_status('job1')
        self.assertEqual(ret['_last_run'], run_time + datetime.timedelta(seconds=delay))

        # Run 6 minutes later
        self.schedule.eval(now=run_time + datetime.timedelta(seconds=delay + 1))
        ret = self.schedule.job_status('job1')
        self.assertEqual(ret['_last_run'], run_time + datetime.timedelta(seconds=delay))



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import
import copy
import logging
import os

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.mixins import SaltReturnAssertsMixin

# Import Salt Testing Libs
from tests.support.mock import MagicMock, patch
import tests.integration as integration

# Import Salt libs
import salt.utils.schedule

from salt.modules.test import ping as ping

try:
    import croniter  # pylint: disable=W0611
    HAS_CRONITER = True
except ImportError:
    HAS_CRONITER = False

log = logging.getLogger(__name__)
ROOT_DIR = os.path.join(integration.TMP, 'schedule-unit-tests')
SOCK_DIR = os.path.join(ROOT_DIR, 'test-socks')

DEFAULT_CONFIG = salt.config.minion_config(None)
DEFAULT_CONFIG['conf_dir'] = ROOT_DIR
DEFAULT_CONFIG['root_dir'] = ROOT_DIR
DEFAULT_CONFIG['sock_dir'] = SOCK_DIR
DEFAULT_CONFIG['pki_dir'] = os.path.join(ROOT_DIR, 'pki')
DEFAULT_CONFIG['cachedir'] = os.path.join(ROOT_DIR, 'cache')


class SchedulerMaxRunningTest(ModuleCase, SaltReturnAssertsMixin):
    '''
    Validate the pkg module
    '''
    def setUp(self):
        with patch('salt.utils.schedule.clean_proc_dir', MagicMock(return_value=None)):
            functions = {'test.ping': ping}
            self.schedule = salt.utils.schedule.Schedule(copy.deepcopy(DEFAULT_CONFIG), functions, returners={})
        self.schedule.opts['loop_interval'] = 1

    def tearDown(self):
        self.schedule.reset()

    def test_maxrunning_minion(self):
        '''
        verify that scheduled job runs
        '''
        self.schedule.opts['__role'] = 'minion'

        job = {
          'schedule': {
            'maxrunning_minion': {
              'function': 'test.ping',
              'seconds': 10,
              'maxrunning': 1
            }
          }
        }

        job_data = {'function': 'test.ping',
                    'run': False,
                    'name': 'maxrunning_minion',
                    'seconds': 10,
                    '_seconds': 10,
                    'jid_include': True,
                    'maxrunning': 1}

        # Add the job to the scheduler
        self.schedule.opts.update(job)

        running_data = [{'fun_args': [],
                         'jid': '20181018165923360935',
                         'schedule': 'maxrunning_minion',
                         'pid': 15338,
                         'fun': 'test.sleep',
                         'id': 'host'}]

        with patch('salt.utils.minion.running',
                   MagicMock(return_value=running_data)):
            with patch('salt.utils.process.os_is_running',
                       MagicMock(return_value=True)):
                ret = self.schedule._check_max_running('test.ping',
                                                       job_data,
                                                       self.schedule.opts)
        self.assertIn('_skip_reason', ret)
        self.assertEqual('maxrunning', ret['_skip_reason'])
        self.assertEqual(False, ret['run'])

    def test_maxrunning_master(self):
        '''
        verify that scheduled job runs
        '''
        self.schedule.opts['__role'] = 'master'

        job = {
          'schedule': {
            'maxrunning_master': {
              'function': 'state.orch',
              'args': ['test.orch_test'],
              'minutes': 1,
              'maxrunning': 1
            }
          }
        }

        job_data = {'function': 'state.orch',
                    'fun_args': ['test.orch_test'],
                    'run': False,
                    'name': 'maxrunning_master',
                    'minutes': 1,
                    'jid_include': True,
                    'maxrunning': 1}

        # Add the job to the scheduler
        self.schedule.opts.update(job)

        running_data = [{'fun_args': ['test.orch_test'],
                         'jid': '20181018165923360935',
                         'schedule': 'maxrunning_master',
                         'pid': 15338,
                         'fun': 'state.orch',
                         'id': 'host'}]

        with patch('salt.utils.master.get_running_jobs',
                   MagicMock(return_value=running_data)):
            with patch('salt.utils.process.os_is_running',
                       MagicMock(return_value=True)):
                ret = self.schedule._check_max_running('state.orch',
                                                       job_data,
                                                       self.schedule.opts)
        self.assertIn('_skip_reason', ret)
        self.assertEqual('maxrunning', ret['_skip_reason'])
        self.assertEqual(False, ret['run'])



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Alexandru Bleotu <alexandru.bleotu@morganstanley.com>`

    Tests functions in salt.utils.vsan
'''

# Import python libraries
from __future__ import absolute_import, print_function, unicode_literals
import logging

# Import Salt testing libraries
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, patch, MagicMock, \
        PropertyMock

# Import Salt libraries
from salt.exceptions import VMwareApiError, VMwareRuntimeError, \
        VMwareObjectRetrievalError
from salt.utils import vsan

try:
    from pyVmomi import vim, vmodl
    HAS_PYVMOMI = True
except ImportError:
    HAS_PYVMOMI = False
HAS_PYVSAN = vsan.HAS_PYVSAN


# Get Logging Started
log = logging.getLogger(__name__)


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_PYVMOMI, 'The \'pyvmomi\' library is missing')
@skipIf(not HAS_PYVSAN, 'The \'vsan\' ext library is missing')
class VsanSupportedTestCase(TestCase):
    '''Tests for salt.utils.vsan.vsan_supported'''

    def test_supported_api_version(self):
        mock_si = MagicMock(content=MagicMock(about=MagicMock()))
        type(mock_si.content.about).apiVersion = \
                PropertyMock(return_value='6.0')
        self.assertTrue(vsan.vsan_supported(mock_si))

    def test_unsupported_api_version(self):
        mock_si = MagicMock(content=MagicMock(about=MagicMock()))
        type(mock_si.content.about).apiVersion = \
                PropertyMock(return_value='5.0')
        self.assertFalse(vsan.vsan_supported(mock_si))

    def test_api_version_raises_no_permission(self):
        exc = vim.fault.NoPermission()
        exc.privilegeId = 'Fake privilege'
        mock_si = MagicMock(content=MagicMock(about=MagicMock()))
        type(mock_si.content.about).apiVersion = PropertyMock(side_effect=exc)
        with self.assertRaises(VMwareApiError) as excinfo:
            vsan.vsan_supported(mock_si)
        self.assertEqual(excinfo.exception.strerror,
                         'Not enough permissions. Required privilege: '
                         'Fake privilege')

    def test_api_version_raises_vim_fault(self):
        exc = vim.fault.VimFault()
        exc.msg = 'VimFault msg'
        mock_si = MagicMock(content=MagicMock(about=MagicMock()))
        type(mock_si.content.about).apiVersion = PropertyMock(side_effect=exc)
        with self.assertRaises(VMwareApiError) as excinfo:
            vsan.vsan_supported(mock_si)
        self.assertEqual(excinfo.exception.strerror, 'VimFault msg')

    def test_api_version_raises_runtime_fault(self):
        exc = vmodl.RuntimeFault()
        exc.msg = 'RuntimeFault msg'
        mock_si = MagicMock(content=MagicMock(about=MagicMock()))
        type(mock_si.content.about).apiVersion = PropertyMock(side_effect=exc)
        with self.assertRaises(VMwareRuntimeError) as excinfo:
            vsan.vsan_supported(mock_si)
        self.assertEqual(excinfo.exception.strerror, 'RuntimeFault msg')


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_PYVMOMI, 'The \'pyvmomi\' library is missing')
@skipIf(not HAS_PYVSAN, 'The \'vsan\' ext library is missing')
class GetVsanClusterConfigSystemTestCase(TestCase, LoaderModuleMockMixin):
    '''Tests for salt.utils.vsan.get_vsan_cluster_config_system'''
    def setup_loader_modules(self):
        return {vsan: {
            '__virtual__': MagicMock(return_value='vsan'),
            'sys': MagicMock(),
            'ssl': MagicMock()}}

    def setUp(self):
        self.mock_si = MagicMock()
        self.mock_ret = MagicMock()
        patches = (('salt.utils.vsan.vsanapiutils.GetVsanVcMos',
                    MagicMock(
                        return_value={'vsan-cluster-config-system':
                                      self.mock_ret})),)
        for mod, mock in patches:
            patcher = patch(mod, mock)
            patcher.start()
            self.addCleanup(patcher.stop)

        type(vsan.sys).version_info = PropertyMock(return_value=(2, 7, 9))
        self.mock_context = MagicMock()
        self.mock_create_default_context = \
                MagicMock(return_value=self.mock_context)
        vsan.ssl.create_default_context = self.mock_create_default_context

    def tearDown(self):
        for attr in ('mock_si', 'mock_ret', 'mock_context',
                     'mock_create_default_context'):
            delattr(self, attr)

    def test_ssl_default_context_loaded(self):
        vsan.get_vsan_cluster_config_system(self.mock_si)
        self.mock_create_default_context.assert_called_once_with()
        self.assertFalse(self.mock_context.check_hostname)
        self.assertEqual(self.mock_context.verify_mode, vsan.ssl.CERT_NONE)

    def test_ssl_default_context_not_loaded(self):
        type(vsan.sys).version_info = PropertyMock(return_value=(2, 7, 8))
        vsan.get_vsan_cluster_config_system(self.mock_si)
        self.assertEqual(self.mock_create_default_context.call_count, 0)

    def test_GetVsanVcMos_call(self):
        mock_get_vsan_vc_mos = MagicMock()
        with patch('salt.utils.vsan.vsanapiutils.GetVsanVcMos',
                   mock_get_vsan_vc_mos):
            vsan.get_vsan_cluster_config_system(self.mock_si)
        mock_get_vsan_vc_mos.assert_called_once_with(self.mock_si._stub,
                                                     context=self.mock_context)

    def test_return(self):
        ret = vsan.get_vsan_cluster_config_system(self.mock_si)
        self.assertEqual(ret, self.mock_ret)


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_PYVMOMI, 'The \'pyvmomi\' library is missing')
@skipIf(not HAS_PYVSAN, 'The \'pyvsan\' bindings are missing')
class GetVsanDiskManagementSystemTestCase(TestCase, LoaderModuleMockMixin):
    '''Tests for salt.utils.vsan.get_vsan_disk_management_system'''
    def setup_loader_modules(self):
        return {vsan: {
            '__virtual__': MagicMock(return_value='vsan'),
            'sys': MagicMock(),
            'ssl': MagicMock()}}

    def setUp(self):
        self.mock_si = MagicMock()
        self.mock_ret = MagicMock()
        patches = (('salt.utils.vsan.vsanapiutils.GetVsanVcMos',
                    MagicMock(
                        return_value={'vsan-disk-management-system':
                                      self.mock_ret})),)
        for mod, mock in patches:
            patcher = patch(mod, mock)
            patcher.start()
            self.addCleanup(patcher.stop)

        type(vsan.sys).version_info = PropertyMock(return_value=(2, 7, 9))
        self.mock_context = MagicMock()
        self.mock_create_default_context = \
                MagicMock(return_value=self.mock_context)
        vsan.ssl.create_default_context = self.mock_create_default_context

    def tearDown(self):
        for attr in ('mock_si', 'mock_ret', 'mock_context',
                     'mock_create_default_context'):
            delattr(self, attr)

    def test_ssl_default_context_loaded(self):
        vsan.get_vsan_disk_management_system(self.mock_si)
        self.mock_create_default_context.assert_called_once_with()
        self.assertFalse(self.mock_context.check_hostname)
        self.assertEqual(self.mock_context.verify_mode, vsan.ssl.CERT_NONE)

    def test_ssl_default_context_not_loaded(self):
        type(vsan.sys).version_info = PropertyMock(return_value=(2, 7, 8))
        vsan.get_vsan_disk_management_system(self.mock_si)
        self.assertEqual(self.mock_create_default_context.call_count, 0)

    def test_GetVsanVcMos_call(self):
        mock_get_vsan_vc_mos = MagicMock()
        with patch('salt.utils.vsan.vsanapiutils.GetVsanVcMos',
                   mock_get_vsan_vc_mos):
            vsan.get_vsan_disk_management_system(self.mock_si)
        mock_get_vsan_vc_mos.assert_called_once_with(self.mock_si._stub,
                                                     context=self.mock_context)

    def test_return(self):
        ret = vsan.get_vsan_disk_management_system(self.mock_si)
        self.assertEqual(ret, self.mock_ret)


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_PYVMOMI, 'The \'pyvmomi\' library is missing')
@skipIf(not HAS_PYVSAN, 'The \'vsan\' ext library is missing')
class GetHostVsanSystemTestCase(TestCase):
    '''Tests for salt.utils.vsan.get_host_vsan_system'''

    def setUp(self):
        self.mock_host_ref = MagicMock()
        self.mock_si = MagicMock()
        self.mock_traversal_spec = MagicMock()
        self.mock_vsan_system = MagicMock()
        patches = (
            ('salt.utils.vmware.get_managed_object_name',
             MagicMock(return_value='fake_hostname')),
            ('salt.utils.vsan.vmodl.query.PropertyCollector.TraversalSpec',
             MagicMock(return_value=self.mock_traversal_spec)),
            ('salt.utils.vmware.get_mors_with_properties',
             MagicMock(return_value=self.mock_traversal_spec)),
            ('salt.utils.vmware.get_mors_with_properties',
             MagicMock(return_value=[{'object': self.mock_vsan_system}])))
        for mod, mock in patches:
            patcher = patch(mod, mock)
            patcher.start()
            self.addCleanup(patcher.stop)

    def test_get_hostname(self):
        mock_get_managed_object_name = MagicMock(return_value='fake_hostname')
        with patch('salt.utils.vmware.get_managed_object_name',
                   mock_get_managed_object_name):
            vsan.get_host_vsan_system(self.mock_si, self.mock_host_ref)
        mock_get_managed_object_name.assert_called_once_with(
            self.mock_host_ref)

    def test_hostname_argument(self):
        mock_get_managed_object_name = MagicMock(return_value='fake_hostname')
        with patch('salt.utils.vmware.get_managed_object_name',
                   MagicMock(return_value='fake_hostname')):
            vsan.get_host_vsan_system(self.mock_si,
                                      self.mock_host_ref,
                                      hostname='passedin_hostname')
        self.assertEqual(mock_get_managed_object_name.call_count, 0)

    def test_traversal_spec(self):
        mock_traversal_spec = MagicMock(return_value=self.mock_traversal_spec)
        with patch(
            'salt.utils.vmware.vmodl.query.PropertyCollector.TraversalSpec',
            mock_traversal_spec):

            vsan.get_host_vsan_system(self.mock_si, self.mock_host_ref)
        mock_traversal_spec.assert_called_once_with(
            path='configManager.vsanSystem',
            type=vim.HostSystem,
            skip=False)

    def test_get_mors_with_properties(self):
        mock_get_mors = \
                MagicMock(return_value=[{'object': self.mock_vsan_system}])
        with patch('salt.utils.vmware.get_mors_with_properties',
                   mock_get_mors):
            vsan.get_host_vsan_system(self.mock_si, self.mock_host_ref)
        mock_get_mors.assert_called_once_with(
            self.mock_si,
            vim.HostVsanSystem,
            property_list=['config.enabled'],
            container_ref=self.mock_host_ref,
            traversal_spec=self.mock_traversal_spec)

    def test_empty_mors_result(self):
        mock_get_mors = MagicMock(return_value=None)
        with patch('salt.utils.vmware.get_mors_with_properties',
                   mock_get_mors):

            with self.assertRaises(VMwareObjectRetrievalError) as excinfo:
                vsan.get_host_vsan_system(self.mock_si, self.mock_host_ref)
        self.assertEqual(excinfo.exception.strerror,
                         'Host\'s \'fake_hostname\' VSAN system was '
                         'not retrieved')

    def test_valid_mors_result(self):
        res = vsan.get_host_vsan_system(self.mock_si, self.mock_host_ref)
        self.assertEqual(res, self.mock_vsan_system)


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_PYVMOMI, 'The \'pyvmomi\' library is missing')
@skipIf(not HAS_PYVSAN, 'The \'vsan\' ext library is missing')
class CreateDiskgroupTestCase(TestCase):
    '''Tests for salt.utils.vsan.create_diskgroup'''
    def setUp(self):
        self.mock_si = MagicMock()
        self.mock_task = MagicMock()
        self.mock_initialise_disk_mapping = \
                MagicMock(return_value=self.mock_task)
        self.mock_vsan_disk_mgmt_system = MagicMock(
            InitializeDiskMappings=self.mock_initialise_disk_mapping)
        self.mock_host_ref = MagicMock()
        self.mock_cache_disk = MagicMock()
        self.mock_cap_disk1 = MagicMock()
        self.mock_cap_disk2 = MagicMock()
        self.mock_spec = MagicMock()
        patches = (
            ('salt.utils.vmware.get_managed_object_name',
             MagicMock(return_value='fake_hostname')),
            ('salt.utils.vsan.vim.VimVsanHostDiskMappingCreationSpec',
             MagicMock(return_value=self.mock_spec)),
            ('salt.utils.vsan._wait_for_tasks', MagicMock()))
        for mod, mock in patches:
            patcher = patch(mod, mock)
            patcher.start()
            self.addCleanup(patcher.stop)

    def test_get_hostname(self):
        mock_get_managed_object_name = MagicMock(return_value='fake_hostname')
        with patch('salt.utils.vmware.get_managed_object_name',
                   mock_get_managed_object_name):
            vsan.create_diskgroup(self.mock_si, self.mock_vsan_disk_mgmt_system,
                                  self.mock_host_ref, self.mock_cache_disk,
                                  [self.mock_cap_disk1, self.mock_cap_disk2])
        mock_get_managed_object_name.assert_called_once_with(
            self.mock_host_ref)

    def test_vsan_spec_all_flash(self):
        self.mock_cap_disk1.ssd = True
        vsan.create_diskgroup(self.mock_si, self.mock_vsan_disk_mgmt_system,
                              self.mock_host_ref, self.mock_cache_disk,
                              [self.mock_cap_disk1, self.mock_cap_disk2])
        self.assertEqual(self.mock_spec.capacityDisks, [self.mock_cap_disk1,
                                                        self.mock_cap_disk2])
        self.assertEqual(self.mock_spec.cacheDisks, [self.mock_cache_disk])
        self.assertEqual(self.mock_spec.creationType, 'allFlash')
        self.assertEqual(self.mock_spec.host, self.mock_host_ref)

    def test_vsan_spec_hybrid(self):
        self.mock_cap_disk1.ssd = False
        vsan.create_diskgroup(self.mock_si, self.mock_vsan_disk_mgmt_system,
                              self.mock_host_ref, self.mock_cache_disk,
                              [self.mock_cap_disk1, self.mock_cap_disk2])
        self.mock_cap_disk1.ssd = False
        self.assertEqual(self.mock_spec.creationType, 'hybrid')

    def test_initialize_disk_mapping(self):
        vsan.create_diskgroup(self.mock_si, self.mock_vsan_disk_mgmt_system,
                              self.mock_host_ref, self.mock_cache_disk,
                              [self.mock_cap_disk1, self.mock_cap_disk2])
        self.mock_initialise_disk_mapping.assert_called_once_with(
            self.mock_spec)

    def test_initialize_disk_mapping_raise_no_permission(self):
        err = vim.fault.NoPermission()
        err.privilegeId = 'Fake privilege'
        self.mock_vsan_disk_mgmt_system.InitializeDiskMappings = \
            MagicMock(side_effect=err)
        with self.assertRaises(VMwareApiError) as excinfo:
            vsan.create_diskgroup(self.mock_si, self.mock_vsan_disk_mgmt_system,
                                  self.mock_host_ref, self.mock_cache_disk,
                                  [self.mock_cap_disk1, self.mock_cap_disk2])
        self.assertEqual(excinfo.exception.strerror,
                         'Not enough permissions. Required privilege: '
                         'Fake privilege')

    def test_initialize_disk_mapping_raise_vim_fault(self):
        err = vim.fault.VimFault()
        err.msg = 'vim_fault'
        self.mock_vsan_disk_mgmt_system.InitializeDiskMappings = \
            MagicMock(side_effect=err)
        with self.assertRaises(VMwareApiError) as excinfo:
            vsan.create_diskgroup(self.mock_si, self.mock_vsan_disk_mgmt_system,
                                  self.mock_host_ref, self.mock_cache_disk,
                                  [self.mock_cap_disk1, self.mock_cap_disk2])
        self.assertEqual(excinfo.exception.strerror, 'vim_fault')

    def test_initialize_disk_mapping_raise_method_not_found(self):
        err = vmodl.fault.MethodNotFound()
        err.method = 'fake_method'
        self.mock_vsan_disk_mgmt_system.InitializeDiskMappings = \
            MagicMock(side_effect=err)
        with self.assertRaises(VMwareRuntimeError) as excinfo:
            vsan.create_diskgroup(self.mock_si, self.mock_vsan_disk_mgmt_system,
                                  self.mock_host_ref, self.mock_cache_disk,
                                  [self.mock_cap_disk1, self.mock_cap_disk2])
        self.assertEqual(excinfo.exception.strerror,
                         'Method \'fake_method\' not found')

    def test_initialize_disk_mapping_raise_runtime_fault(self):
        err = vmodl.RuntimeFault()
        err.msg = 'runtime_fault'
        self.mock_vsan_disk_mgmt_system.InitializeDiskMappings = \
            MagicMock(side_effect=err)
        with self.assertRaises(VMwareRuntimeError) as excinfo:
            vsan.create_diskgroup(self.mock_si, self.mock_vsan_disk_mgmt_system,
                                  self.mock_host_ref, self.mock_cache_disk,
                                  [self.mock_cap_disk1, self.mock_cap_disk2])
        self.assertEqual(excinfo.exception.strerror, 'runtime_fault')

    def test__wait_for_tasks(self):
        mock___wait_for_tasks = MagicMock()
        with patch('salt.utils.vsan._wait_for_tasks',
                   mock___wait_for_tasks):
            vsan.create_diskgroup(self.mock_si, self.mock_vsan_disk_mgmt_system,
                                  self.mock_host_ref, self.mock_cache_disk,
                                  [self.mock_cap_disk1, self.mock_cap_disk2])
        mock___wait_for_tasks.assert_called_once_with(
            [self.mock_task], self.mock_si)

    def test_result(self):
        res = vsan.create_diskgroup(self.mock_si,
                                    self.mock_vsan_disk_mgmt_system,
                                    self.mock_host_ref, self.mock_cache_disk,
                                    [self.mock_cap_disk1, self.mock_cap_disk2])
        self.assertTrue(res)


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_PYVMOMI, 'The \'pyvmomi\' library is missing')
@skipIf(not HAS_PYVSAN, 'The \'vsan\' ext library is missing')
class AddCapacityToDiskGroupTestCase(TestCase):
    '''Tests for salt.utils.vsan.add_capacity_to_diskgroup'''
    def setUp(self):
        self.mock_si = MagicMock()
        self.mock_task = MagicMock()
        self.mock_initialise_disk_mapping = \
                MagicMock(return_value=self.mock_task)
        self.mock_vsan_disk_mgmt_system = MagicMock(
            InitializeDiskMappings=self.mock_initialise_disk_mapping)
        self.mock_host_ref = MagicMock()
        self.mock_cache_disk = MagicMock()
        self.mock_diskgroup = MagicMock(ssd=self.mock_cache_disk)
        self.mock_cap_disk1 = MagicMock()
        self.mock_cap_disk2 = MagicMock()
        self.mock_spec = MagicMock()
        patches = (
            ('salt.utils.vmware.get_managed_object_name',
             MagicMock(return_value='fake_hostname')),
            ('salt.utils.vsan.vim.VimVsanHostDiskMappingCreationSpec',
             MagicMock(return_value=self.mock_spec)),
            ('salt.utils.vsan._wait_for_tasks', MagicMock()))
        for mod, mock in patches:
            patcher = patch(mod, mock)
            patcher.start()
            self.addCleanup(patcher.stop)

    def test_get_hostname(self):
        mock_get_managed_object_name = MagicMock(return_value='fake_hostname')
        with patch('salt.utils.vmware.get_managed_object_name',
                   mock_get_managed_object_name):
            vsan.add_capacity_to_diskgroup(
                self.mock_si, self.mock_vsan_disk_mgmt_system,
                self.mock_host_ref, self.mock_diskgroup,
                [self.mock_cap_disk1, self.mock_cap_disk2])
        mock_get_managed_object_name.assert_called_once_with(
            self.mock_host_ref)

    def test_vsan_spec_all_flash(self):
        self.mock_cap_disk1.ssd = True
        vsan.add_capacity_to_diskgroup(
            self.mock_si, self.mock_vsan_disk_mgmt_system,
            self.mock_host_ref, self.mock_diskgroup,
            [self.mock_cap_disk1, self.mock_cap_disk2])
        self.assertEqual(self.mock_spec.capacityDisks, [self.mock_cap_disk1,
                                                        self.mock_cap_disk2])
        self.assertEqual(self.mock_spec.cacheDisks, [self.mock_cache_disk])
        self.assertEqual(self.mock_spec.creationType, 'allFlash')
        self.assertEqual(self.mock_spec.host, self.mock_host_ref)

    def test_vsan_spec_hybrid(self):
        self.mock_cap_disk1.ssd = False
        vsan.add_capacity_to_diskgroup(
            self.mock_si, self.mock_vsan_disk_mgmt_system,
            self.mock_host_ref, self.mock_diskgroup,
            [self.mock_cap_disk1, self.mock_cap_disk2])
        self.mock_cap_disk1.ssd = False
        self.assertEqual(self.mock_spec.creationType, 'hybrid')

    def test_initialize_disk_mapping(self):
        vsan.add_capacity_to_diskgroup(
            self.mock_si, self.mock_vsan_disk_mgmt_system,
            self.mock_host_ref, self.mock_diskgroup,
            [self.mock_cap_disk1, self.mock_cap_disk2])
        self.mock_initialise_disk_mapping.assert_called_once_with(
            self.mock_spec)

    def test_initialize_disk_mapping_raise_no_permission(self):
        err = vim.fault.NoPermission()
        err.privilegeId = 'Fake privilege'
        self.mock_vsan_disk_mgmt_system.InitializeDiskMappings = \
            MagicMock(side_effect=err)
        with self.assertRaises(VMwareApiError) as excinfo:
            vsan.add_capacity_to_diskgroup(
                self.mock_si, self.mock_vsan_disk_mgmt_system,
                self.mock_host_ref, self.mock_diskgroup,
                [self.mock_cap_disk1, self.mock_cap_disk2])
        self.assertEqual(excinfo.exception.strerror,
                         'Not enough permissions. Required privilege: '
                         'Fake privilege')

    def test_initialize_disk_mapping_raise_vim_fault(self):
        err = vim.fault.VimFault()
        err.msg = 'vim_fault'
        self.mock_vsan_disk_mgmt_system.InitializeDiskMappings = \
            MagicMock(side_effect=err)
        with self.assertRaises(VMwareApiError) as excinfo:
            vsan.add_capacity_to_diskgroup(
                self.mock_si, self.mock_vsan_disk_mgmt_system,
                self.mock_host_ref, self.mock_diskgroup,
                [self.mock_cap_disk1, self.mock_cap_disk2])
        self.assertEqual(excinfo.exception.strerror, 'vim_fault')

    def test_initialize_disk_mapping_raise_method_not_found(self):
        err = vmodl.fault.MethodNotFound()
        err.method = 'fake_method'
        self.mock_vsan_disk_mgmt_system.InitializeDiskMappings = \
            MagicMock(side_effect=err)
        with self.assertRaises(VMwareRuntimeError) as excinfo:
            vsan.add_capacity_to_diskgroup(
                self.mock_si, self.mock_vsan_disk_mgmt_system,
                self.mock_host_ref, self.mock_diskgroup,
                [self.mock_cap_disk1, self.mock_cap_disk2])
        self.assertEqual(excinfo.exception.strerror,
                         'Method \'fake_method\' not found')

    def test_initialize_disk_mapping_raise_runtime_fault(self):
        err = vmodl.RuntimeFault()
        err.msg = 'runtime_fault'
        self.mock_vsan_disk_mgmt_system.InitializeDiskMappings = \
            MagicMock(side_effect=err)
        with self.assertRaises(VMwareRuntimeError) as excinfo:
            vsan.add_capacity_to_diskgroup(
                self.mock_si, self.mock_vsan_disk_mgmt_system,
                self.mock_host_ref, self.mock_diskgroup,
                [self.mock_cap_disk1, self.mock_cap_disk2])
        self.assertEqual(excinfo.exception.strerror, 'runtime_fault')

    def test__wait_for_tasks(self):
        mock___wait_for_tasks = MagicMock()
        with patch('salt.utils.vsan._wait_for_tasks',
                   mock___wait_for_tasks):
            vsan.add_capacity_to_diskgroup(
                self.mock_si, self.mock_vsan_disk_mgmt_system,
                self.mock_host_ref, self.mock_diskgroup,
                [self.mock_cap_disk1, self.mock_cap_disk2])
        mock___wait_for_tasks.assert_called_once_with(
            [self.mock_task], self.mock_si)

    def test_result(self):
        res = vsan.add_capacity_to_diskgroup(
            self.mock_si, self.mock_vsan_disk_mgmt_system,
            self.mock_host_ref, self.mock_diskgroup,
            [self.mock_cap_disk1, self.mock_cap_disk2])
        self.assertTrue(res)


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_PYVMOMI, 'The \'pyvmomi\' library is missing')
@skipIf(not HAS_PYVSAN, 'The \'vsan\' ext library is missing')
class RemoveCapacityFromDiskGroup(TestCase):
    '''Tests for salt.utils.vsan.remove_capacity_from_diskgroup'''
    def setUp(self):
        self.mock_si = MagicMock()
        self.mock_task = MagicMock()
        self.mock_remove_disk = \
                MagicMock(return_value=self.mock_task)
        self.mock_host_vsan_system = MagicMock(
            RemoveDisk_Task=self.mock_remove_disk)
        self.mock_host_ref = MagicMock()
        self.mock_cache_disk = MagicMock()
        self.mock_diskgroup = MagicMock(ssd=self.mock_cache_disk)
        self.mock_cap_disk1 = MagicMock()
        self.mock_cap_disk2 = MagicMock()
        self.mock_spec = MagicMock()
        patches = (
            ('salt.utils.vmware.get_managed_object_name',
             MagicMock(return_value='fake_hostname')),
            ('salt.utils.vsan.get_host_vsan_system',
             MagicMock(return_value=self.mock_host_vsan_system)),
            ('salt.utils.vsan.vim.HostMaintenanceSpec',
             MagicMock(return_value=self.mock_spec)),
            ('salt.utils.vsan.vim.VsanHostDecommissionMode', MagicMock()),
            ('salt.utils.vmware.wait_for_task', MagicMock()))
        for mod, mock in patches:
            patcher = patch(mod, mock)
            patcher.start()
            self.addCleanup(patcher.stop)

    def test_get_hostname(self):
        mock_get_managed_object_name = MagicMock(return_value='fake_hostname')
        with patch('salt.utils.vmware.get_managed_object_name',
                   mock_get_managed_object_name):
            vsan.remove_capacity_from_diskgroup(
                self.mock_si, self.mock_host_ref, self.mock_diskgroup,
                [self.mock_cap_disk1, self.mock_cap_disk2])
        mock_get_managed_object_name.assert_called_once_with(
            self.mock_host_ref)

    def test_maintenance_mode_evacuate_all_data(self):
        vsan.remove_capacity_from_diskgroup(
            self.mock_si, self.mock_host_ref, self.mock_diskgroup,
            [self.mock_cap_disk1, self.mock_cap_disk2])
        self.assertEqual(self.mock_spec.vsanMode.objectAction,
                         vim.VsanHostDecommissionModeObjectAction.evacuateAllData)

    def test_maintenance_mode_no_action(self):
        vsan.remove_capacity_from_diskgroup(
            self.mock_si, self.mock_host_ref, self.mock_diskgroup,
            [self.mock_cap_disk1, self.mock_cap_disk2],
            data_evacuation=False)
        self.assertEqual(self.mock_spec.vsanMode.objectAction,
                         vim.VsanHostDecommissionModeObjectAction.noAction)

    def test_remove_disk(self):
        vsan.remove_capacity_from_diskgroup(
            self.mock_si, self.mock_host_ref, self.mock_diskgroup,
            [self.mock_cap_disk1, self.mock_cap_disk2])
        self.mock_remove_disk.assert_called_once_with(
            disk=[self.mock_cap_disk1, self.mock_cap_disk2],
            maintenanceSpec=self.mock_spec)

    def test_remove_disk_raise_no_permission(self):
        err = vim.fault.NoPermission()
        err.privilegeId = 'Fake privilege'
        self.mock_host_vsan_system.RemoveDisk_Task = \
            MagicMock(side_effect=err)
        with self.assertRaises(VMwareApiError) as excinfo:
            vsan.remove_capacity_from_diskgroup(
                self.mock_si, self.mock_host_ref, self.mock_diskgroup,
                [self.mock_cap_disk1, self.mock_cap_disk2])
        self.assertEqual(excinfo.exception.strerror,
                         'Not enough permissions. Required privilege: '
                         'Fake privilege')

    def test_remove_disk_raise_vim_fault(self):
        err = vim.fault.VimFault()
        err.msg = 'vim_fault'
        self.mock_host_vsan_system.RemoveDisk_Task = \
            MagicMock(side_effect=err)
        with self.assertRaises(VMwareApiError) as excinfo:
            vsan.remove_capacity_from_diskgroup(
                self.mock_si, self.mock_host_ref, self.mock_diskgroup,
                [self.mock_cap_disk1, self.mock_cap_disk2])
        self.assertEqual(excinfo.exception.strerror, 'vim_fault')

    def test_remove_disk_raise_runtime_fault(self):
        err = vmodl.RuntimeFault()
        err.msg = 'runtime_fault'
        self.mock_host_vsan_system.RemoveDisk_Task = \
            MagicMock(side_effect=err)
        with self.assertRaises(VMwareRuntimeError) as excinfo:
            vsan.remove_capacity_from_diskgroup(
                self.mock_si, self.mock_host_ref, self.mock_diskgroup,
                [self.mock_cap_disk1, self.mock_cap_disk2])
        self.assertEqual(excinfo.exception.strerror, 'runtime_fault')

    def test_wait_for_tasks(self):
        mock_wait_for_task = MagicMock()
        with patch('salt.utils.vmware.wait_for_task',
                   mock_wait_for_task):
            vsan.remove_capacity_from_diskgroup(
                self.mock_si, self.mock_host_ref, self.mock_diskgroup,
                [self.mock_cap_disk1, self.mock_cap_disk2])
        mock_wait_for_task.assert_called_once_with(
            self.mock_task, 'fake_hostname', 'remove_capacity')

    def test_result(self):
        res = vsan.remove_capacity_from_diskgroup(
            self.mock_si, self.mock_host_ref, self.mock_diskgroup,
            [self.mock_cap_disk1, self.mock_cap_disk2])
        self.assertTrue(res)


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_PYVMOMI, 'The \'pyvmomi\' library is missing')
@skipIf(not HAS_PYVSAN, 'The \'vsan\' ext library is missing')
class RemoveDiskgroup(TestCase):
    '''Tests for salt.utils.vsan.remove_diskgroup'''
    def setUp(self):
        self.mock_si = MagicMock()
        self.mock_task = MagicMock()
        self.mock_remove_disk_mapping = \
                MagicMock(return_value=self.mock_task)
        self.mock_host_vsan_system = MagicMock(
            RemoveDiskMapping_Task=self.mock_remove_disk_mapping)
        self.mock_host_ref = MagicMock()
        self.mock_cache_disk = MagicMock()
        self.mock_diskgroup = MagicMock(ssd=self.mock_cache_disk)
        self.mock_cap_disk1 = MagicMock()
        self.mock_cap_disk2 = MagicMock()
        self.mock_spec = MagicMock()
        patches = (
            ('salt.utils.vmware.get_managed_object_name',
             MagicMock(return_value='fake_hostname')),
            ('salt.utils.vsan.get_host_vsan_system',
             MagicMock(return_value=self.mock_host_vsan_system)),
            ('salt.utils.vsan.vim.HostMaintenanceSpec',
             MagicMock(return_value=self.mock_spec)),
            ('salt.utils.vsan.vim.VsanHostDecommissionMode', MagicMock()),
            ('salt.utils.vmware.wait_for_task', MagicMock()))
        for mod, mock in patches:
            patcher = patch(mod, mock)
            patcher.start()
            self.addCleanup(patcher.stop)

    def test_get_hostname(self):
        mock_get_managed_object_name = MagicMock(return_value='fake_hostname')
        with patch('salt.utils.vmware.get_managed_object_name',
                   mock_get_managed_object_name):
            vsan.remove_diskgroup(
                self.mock_si, self.mock_host_ref, self.mock_diskgroup)
        mock_get_managed_object_name.assert_called_once_with(
            self.mock_host_ref)

    def test_maintenance_mode_evacuate_all_data(self):
        vsan.remove_diskgroup(
            self.mock_si, self.mock_host_ref, self.mock_diskgroup)
        vsan.remove_capacity_from_diskgroup(
            self.mock_si, self.mock_host_ref, self.mock_diskgroup,
            [self.mock_cap_disk1, self.mock_cap_disk2])
        self.assertEqual(self.mock_spec.vsanMode.objectAction,
                         vim.VsanHostDecommissionModeObjectAction.evacuateAllData)

    def test_maintenance_mode_no_action(self):
        vsan.remove_diskgroup(
            self.mock_si, self.mock_host_ref, self.mock_diskgroup)
        vsan.remove_capacity_from_diskgroup(
            self.mock_si, self.mock_host_ref, self.mock_diskgroup,
            [self.mock_cap_disk1, self.mock_cap_disk2],
            data_evacuation=False)
        self.assertEqual(self.mock_spec.vsanMode.objectAction,
                         vim.VsanHostDecommissionModeObjectAction.noAction)

    def test_remove_disk_mapping(self):
        vsan.remove_diskgroup(
            self.mock_si, self.mock_host_ref, self.mock_diskgroup)
        vsan.remove_capacity_from_diskgroup(
            self.mock_si, self.mock_host_ref, self.mock_diskgroup,
            [self.mock_cap_disk1, self.mock_cap_disk2])
        self.mock_remove_disk_mapping.assert_called_once_with(
            mapping=[self.mock_diskgroup],
            maintenanceSpec=self.mock_spec)

    def test_remove_disk_mapping_raise_no_permission(self):
        vsan.remove_diskgroup(
            self.mock_si, self.mock_host_ref, self.mock_diskgroup)
        err = vim.fault.NoPermission()
        err.privilegeId = 'Fake privilege'
        self.mock_host_vsan_system.RemoveDiskMapping_Task = \
            MagicMock(side_effect=err)
        with self.assertRaises(VMwareApiError) as excinfo:
            vsan.remove_diskgroup(
                self.mock_si, self.mock_host_ref, self.mock_diskgroup)
        self.assertEqual(excinfo.exception.strerror,
                         'Not enough permissions. Required privilege: '
                         'Fake privilege')

    def test_remove_disk_mapping_raise_vim_fault(self):
        err = vim.fault.VimFault()
        err.msg = 'vim_fault'
        self.mock_host_vsan_system.RemoveDiskMapping_Task = \
            MagicMock(side_effect=err)
        with self.assertRaises(VMwareApiError) as excinfo:
            vsan.remove_diskgroup(
                self.mock_si, self.mock_host_ref, self.mock_diskgroup)
        self.assertEqual(excinfo.exception.strerror, 'vim_fault')

    def test_remove_disk_mapping_raise_runtime_fault(self):
        err = vmodl.RuntimeFault()
        err.msg = 'runtime_fault'
        self.mock_host_vsan_system.RemoveDiskMapping_Task = \
            MagicMock(side_effect=err)
        with self.assertRaises(VMwareRuntimeError) as excinfo:
            vsan.remove_diskgroup(
                self.mock_si, self.mock_host_ref, self.mock_diskgroup)
        self.assertEqual(excinfo.exception.strerror, 'runtime_fault')

    def test_wait_for_tasks(self):
        mock_wait_for_task = MagicMock()
        with patch('salt.utils.vmware.wait_for_task',
                   mock_wait_for_task):
            vsan.remove_diskgroup(
                self.mock_si, self.mock_host_ref, self.mock_diskgroup)
        mock_wait_for_task.assert_called_once_with(
            self.mock_task, 'fake_hostname', 'remove_diskgroup')

    def test_result(self):
        res = vsan.remove_diskgroup(
            self.mock_si, self.mock_host_ref, self.mock_diskgroup)
        self.assertTrue(res)


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_PYVMOMI, 'The \'pyvmomi\' library is missing')
@skipIf(not HAS_PYVSAN, 'The \'vsan\' ext library is missing')
class GetClusterVsanInfoTestCase(TestCase, LoaderModuleMockMixin):
    '''Tests for salt.utils.vsan.get_cluster_vsan_info'''
    def setup_loader_modules(self):
        return {vsan: {
            '__virtual__': MagicMock(return_value='vsan')}}

    def setUp(self):
        self.mock_cl_ref = MagicMock()
        self.mock_si = MagicMock()
        patches = (
            ('salt.utils.vmware.get_managed_object_name', MagicMock()),
            ('salt.utils.vmware.get_service_instance_from_managed_object',
             MagicMock(return_value=self.mock_si)),
            ('salt.utils.vsan.get_vsan_cluster_config_system', MagicMock()))
        for mod, mock in patches:
            patcher = patch(mod, mock)
            patcher.start()
            self.addCleanup(patcher.stop)

    def tearDown(self):
        for attr in ('mock_si', 'mock_cl_ref'):
            delattr(self, attr)

    def test_get_managed_object_name_call(self):
        mock_get_managed_object_name = MagicMock()
        with patch('salt.utils.vmware.get_managed_object_name',
                   mock_get_managed_object_name):
            vsan.get_cluster_vsan_info(self.mock_cl_ref)
        mock_get_managed_object_name.assert_called_once_with(self.mock_cl_ref)

    def test_get_vsan_cluster_config_system_call(self):
        mock_get_vsan_cl_syst = MagicMock()
        with patch('salt.utils.vsan.get_vsan_cluster_config_system',
                   mock_get_vsan_cl_syst):
            vsan.get_cluster_vsan_info(self.mock_cl_ref)
        mock_get_vsan_cl_syst.assert_called_once_with(self.mock_si)

    def test_VsanClusterGetConfig_call(self):
        mock_vsan_sys = MagicMock()
        with patch('salt.utils.vsan.get_vsan_cluster_config_system',
                   MagicMock(return_value=mock_vsan_sys)):
            vsan.get_cluster_vsan_info(self.mock_cl_ref)
        mock_vsan_sys.VsanClusterGetConfig.assert_called_once_with(
            self.mock_cl_ref)

    def test_VsanClusterGetConfig_raises_no_permission(self):
        exc = vim.fault.NoPermission()
        exc.privilegeId = 'Fake privilege'
        with patch('salt.utils.vsan.get_vsan_cluster_config_system',
                   MagicMock(return_value=MagicMock(
                       VsanClusterGetConfig=MagicMock(side_effect=exc)))):
            with self.assertRaises(VMwareApiError) as excinfo:
                vsan.get_cluster_vsan_info(self.mock_cl_ref)
        self.assertEqual(excinfo.exception.strerror,
                         'Not enough permissions. Required privilege: '
                         'Fake privilege')

    def test_VsanClusterGetConfig_raises_vim_fault(self):
        exc = vim.fault.VimFault()
        exc.msg = 'VimFault msg'
        with patch('salt.utils.vsan.get_vsan_cluster_config_system',
                   MagicMock(return_value=MagicMock(
                       VsanClusterGetConfig=MagicMock(side_effect=exc)))):
            with self.assertRaises(VMwareApiError) as excinfo:
                vsan.get_cluster_vsan_info(self.mock_cl_ref)
        self.assertEqual(excinfo.exception.strerror, 'VimFault msg')

    def test_VsanClusterGetConfig_raises_runtime_fault(self):
        exc = vmodl.RuntimeFault()
        exc.msg = 'RuntimeFault msg'
        with patch('salt.utils.vsan.get_vsan_cluster_config_system',
                   MagicMock(return_value=MagicMock(
                       VsanClusterGetConfig=MagicMock(side_effect=exc)))):
            with self.assertRaises(VMwareRuntimeError) as excinfo:
                vsan.get_cluster_vsan_info(self.mock_cl_ref)
        self.assertEqual(excinfo.exception.strerror, 'RuntimeFault msg')


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_PYVMOMI, 'The \'pyvmomi\' library is missing')
@skipIf(not HAS_PYVSAN, 'The \'vsan\' ext library is missing')
class ReconfigureClusterVsanTestCase(TestCase):
    '''Tests for salt.utils.vsan.reconfigure_cluster_vsan'''
    def setUp(self):
        self.mock_si = MagicMock()
        self.mock_task = MagicMock()
        self.mock_cl_reconf = MagicMock(return_value=self.mock_task)
        self.mock_get_vsan_conf_sys = MagicMock(
            return_value=MagicMock(VsanClusterReconfig=self.mock_cl_reconf))
        self.mock_cl_ref = MagicMock()
        self.mock_cl_vsan_spec = MagicMock()
        patches = (
            ('salt.utils.vmware.get_managed_object_name', MagicMock()),
            ('salt.utils.vmware.get_service_instance_from_managed_object',
             MagicMock(return_value=self.mock_si)),
            ('salt.utils.vsan.get_vsan_cluster_config_system',
             self.mock_get_vsan_conf_sys),
            ('salt.utils.vsan._wait_for_tasks', MagicMock()))
        for mod, mock in patches:
            patcher = patch(mod, mock)
            patcher.start()
            self.addCleanup(patcher.stop)

    def tearDown(self):
        for attr in ('mock_si', 'mock_cl_reconf', 'mock_get_vsan_conf_sys',
                     'mock_cl_ref', 'mock_cl_vsan_spec', 'mock_task'):
            delattr(self, attr)

    def test_get_cluster_name_call(self):
        get_managed_object_name_mock = MagicMock()
        with patch('salt.utils.vmware.get_managed_object_name',
                   get_managed_object_name_mock):

            vsan.reconfigure_cluster_vsan(self.mock_cl_ref,
                                          self.mock_cl_vsan_spec)
        get_managed_object_name_mock.assert_called_once_with(
            self.mock_cl_ref)

    def test_get_service_instance_call(self):
        get_service_instance_from_managed_object_mock = MagicMock()
        with patch(
            'salt.utils.vmware.get_service_instance_from_managed_object',
            get_service_instance_from_managed_object_mock):

            vsan.reconfigure_cluster_vsan(self.mock_cl_ref,
                                          self.mock_cl_vsan_spec)
        get_service_instance_from_managed_object_mock.assert_called_once_with(
            self.mock_cl_ref)

    def test_get_vsan_cluster_config_system_call(self):
        vsan.reconfigure_cluster_vsan(self.mock_cl_ref,
                                      self.mock_cl_vsan_spec)
        self.mock_get_vsan_conf_sys.assert_called_once_with(self.mock_si)

    def test_cluster_reconfig_call(self):
        vsan.reconfigure_cluster_vsan(self.mock_cl_ref,
                                      self.mock_cl_vsan_spec)
        self.mock_cl_reconf.assert_called_once_with(
            self.mock_cl_ref, self.mock_cl_vsan_spec)

    def test_cluster_reconfig_raises_no_permission(self):
        exc = vim.fault.NoPermission()
        exc.privilegeId = 'Fake privilege'
        with patch('salt.utils.vsan.get_vsan_cluster_config_system',
                   MagicMock(return_value=MagicMock(
                       VsanClusterReconfig=MagicMock(side_effect=exc)))):
            with self.assertRaises(VMwareApiError) as excinfo:
                vsan.reconfigure_cluster_vsan(self.mock_cl_ref,
                                              self.mock_cl_vsan_spec)
        self.assertEqual(excinfo.exception.strerror,
                         'Not enough permissions. Required privilege: '
                         'Fake privilege')

    def test_cluster_reconfig_raises_vim_fault(self):
        exc = vim.fault.VimFault()
        exc.msg = 'VimFault msg'
        with patch('salt.utils.vsan.get_vsan_cluster_config_system',
                   MagicMock(return_value=MagicMock(
                       VsanClusterReconfig=MagicMock(side_effect=exc)))):
            with self.assertRaises(VMwareApiError) as excinfo:
                vsan.reconfigure_cluster_vsan(self.mock_cl_ref,
                                              self.mock_cl_vsan_spec)
        self.assertEqual(excinfo.exception.strerror, 'VimFault msg')

    def test_cluster_reconfig_raises_vmodl_runtime_error(self):
        exc = vmodl.RuntimeFault()
        exc.msg = 'VimRuntime msg'
        with patch('salt.utils.vsan.get_vsan_cluster_config_system',
                   MagicMock(return_value=MagicMock(
                       VsanClusterReconfig=MagicMock(side_effect=exc)))):
            with self.assertRaises(VMwareRuntimeError) as excinfo:
                vsan.reconfigure_cluster_vsan(self.mock_cl_ref,
                                              self.mock_cl_vsan_spec)
        self.assertEqual(excinfo.exception.strerror, 'VimRuntime msg')

    def test__wait_for_tasks_call(self):
        mock_wait_for_tasks = MagicMock()
        with patch('salt.utils.vsan._wait_for_tasks', mock_wait_for_tasks):
            vsan.reconfigure_cluster_vsan(self.mock_cl_ref,
                                          self.mock_cl_vsan_spec)
        mock_wait_for_tasks.assert_called_once_with([self.mock_task],
                                                    self.mock_si)


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_PYVMOMI, 'The \'pyvmomi\' library is missing')
@skipIf(not HAS_PYVSAN, 'The \'vsan\' ext library is missing')
class _WaitForTasks(TestCase, LoaderModuleMockMixin):
    '''Tests for salt.utils.vsan._wait_for_tasks'''
    def setup_loader_modules(self):
        return {vsan: {
            '__virtual__': MagicMock(return_value='vsan')}}

    def setUp(self):
        self.mock_si = MagicMock()
        self.mock_tasks = MagicMock()
        patches = (('salt.utils.vsan.vsanapiutils.WaitForTasks', MagicMock()),)
        for mod, mock in patches:
            patcher = patch(mod, mock)
            patcher.start()
            self.addCleanup(patcher.stop)

    def tearDown(self):
        for attr in ('mock_si', 'mock_tasks'):
            delattr(self, attr)

    def test_wait_for_tasks_call(self):
        mock_wait_for_tasks = MagicMock()
        with patch('salt.utils.vsan.vsanapiutils.WaitForTasks',
                   mock_wait_for_tasks):
            vsan._wait_for_tasks(self.mock_tasks, self.mock_si)
        mock_wait_for_tasks.assert_called_once_with(self.mock_tasks,
                                                    self.mock_si)

    def test_wait_for_tasks_raises_no_permission(self):
        exc = vim.fault.NoPermission()
        exc.privilegeId = 'Fake privilege'
        with patch('salt.utils.vsan.vsanapiutils.WaitForTasks',
                   MagicMock(side_effect=exc)):
            with self.assertRaises(VMwareApiError) as excinfo:
                vsan._wait_for_tasks(self.mock_tasks, self.mock_si)
        self.assertEqual(excinfo.exception.strerror,
                         'Not enough permissions. Required privilege: '
                         'Fake privilege')

    def test_wait_for_tasks_raises_vim_fault(self):
        exc = vim.fault.VimFault()
        exc.msg = 'VimFault msg'
        with patch('salt.utils.vsan.vsanapiutils.WaitForTasks',
                   MagicMock(side_effect=exc)):
            with self.assertRaises(VMwareApiError) as excinfo:
                vsan._wait_for_tasks(self.mock_tasks, self.mock_si)
        self.assertEqual(excinfo.exception.strerror, 'VimFault msg')

    def test_wait_for_tasks_raises_vmodl_runtime_error(self):
        exc = vmodl.RuntimeFault()
        exc.msg = 'VimRuntime msg'
        with patch('salt.utils.vsan.vsanapiutils.WaitForTasks',
                   MagicMock(side_effect=exc)):
            with self.assertRaises(VMwareRuntimeError) as excinfo:
                vsan._wait_for_tasks(self.mock_tasks, self.mock_si)
        self.assertEqual(excinfo.exception.strerror, 'VimRuntime msg')



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import os
import shutil

# Import Salt Testing Libs
from tests.support.case import SSHCase
from tests.support.unit import skipIf

# Import Salt Libs
import salt.utils.platform


@skipIf(salt.utils.platform.is_windows(), 'salt-ssh not available on Windows')
class SSHMineTest(SSHCase):
    '''
    testing salt-ssh with mine
    '''
    def test_ssh_mine_get(self):
        '''
        test salt-ssh with mine
        '''
        ret = self.run_function('mine.get', ['localhost test.arg'], wipe=False)
        self.assertEqual(ret['localhost']['args'], ['itworked'])

    def tearDown(self):
        '''
        make sure to clean up any old ssh directories
        '''
        salt_dir = self.run_function('config.get', ['thin_dir'], wipe=False)
        if os.path.exists(salt_dir):
            shutil.rmtree(salt_dir)



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import

# Import Salt libs
import salt.config
import salt.master

# Import Salt Testing Libs
from tests.support.unit import TestCase
from tests.support.mock import (
    patch,
    MagicMock,
)


class ClearFuncsTestCase(TestCase):
    '''
    TestCase for salt.master.ClearFuncs class
    '''

    def setUp(self):
        opts = salt.config.master_config(None)
        self.clear_funcs = salt.master.ClearFuncs(opts, {})

    # runner tests

    def test_runner_token_not_authenticated(self):
        '''
        Asserts that a TokenAuthenticationError is returned when the token can't authenticate.
        '''
        mock_ret = {'error': {'name': 'TokenAuthenticationError',
                              'message': 'Authentication failure of type "token" occurred.'}}
        ret = self.clear_funcs.runner({'token': 'asdfasdfasdfasdf'})
        self.assertDictEqual(mock_ret, ret)

    def test_runner_token_authorization_error(self):
        '''
        Asserts that a TokenAuthenticationError is returned when the token authenticates, but is
        not authorized.
        '''
        token = 'asdfasdfasdfasdf'
        clear_load = {'token': token, 'fun': 'test.arg'}
        mock_token = {'token': token, 'eauth': 'foo', 'name': 'test'}
        mock_ret = {'error': {'name': 'TokenAuthenticationError',
                               'message': 'Authentication failure of type "token" occurred '
                                          'for user test.'}}

        with patch('salt.auth.LoadAuth.authenticate_token', MagicMock(return_value=mock_token)), \
             patch('salt.auth.LoadAuth.get_auth_list', MagicMock(return_value=[])):
            ret = self.clear_funcs.runner(clear_load)

        self.assertDictEqual(mock_ret, ret)

    def test_runner_token_salt_invocation_error(self):
        '''
        Asserts that a SaltInvocationError is returned when the token authenticates, but the
        command is malformed.
        '''
        token = 'asdfasdfasdfasdf'
        clear_load = {'token': token, 'fun': 'badtestarg'}
        mock_token = {'token': token, 'eauth': 'foo', 'name': 'test'}
        mock_ret = {'error': {'name': 'SaltInvocationError',
                              'message': 'A command invocation error occurred: Check syntax.'}}

        with patch('salt.auth.LoadAuth.authenticate_token', MagicMock(return_value=mock_token)), \
             patch('salt.auth.LoadAuth.get_auth_list', MagicMock(return_value=['testing'])):
            ret = self.clear_funcs.runner(clear_load)

        self.assertDictEqual(mock_ret, ret)

    def test_runner_eauth_not_authenticated(self):
        '''
        Asserts that an EauthAuthenticationError is returned when the user can't authenticate.
        '''
        mock_ret = {'error': {'name': 'EauthAuthenticationError',
                              'message': 'Authentication failure of type "eauth" occurred for '
                                         'user UNKNOWN.'}}
        ret = self.clear_funcs.runner({'eauth': 'foo'})
        self.assertDictEqual(mock_ret, ret)

    def test_runner_eauth_authorization_error(self):
        '''
        Asserts that an EauthAuthenticationError is returned when the user authenticates, but is
        not authorized.
        '''
        clear_load = {'eauth': 'foo', 'username': 'test', 'fun': 'test.arg'}
        mock_ret = {'error': {'name': 'EauthAuthenticationError',
                              'message': 'Authentication failure of type "eauth" occurred for '
                                         'user test.'}}
        with patch('salt.auth.LoadAuth.authenticate_eauth', MagicMock(return_value=True)), \
             patch('salt.auth.LoadAuth.get_auth_list', MagicMock(return_value=[])):
            ret = self.clear_funcs.runner(clear_load)

        self.assertDictEqual(mock_ret, ret)

    def test_runner_eauth_salt_invocation_error(self):
        '''
        Asserts that an EauthAuthenticationError is returned when the user authenticates, but the
        command is malformed.
        '''
        clear_load = {'eauth': 'foo', 'username': 'test', 'fun': 'bad.test.arg.func'}
        mock_ret = {'error': {'name': 'SaltInvocationError',
                              'message': 'A command invocation error occurred: Check syntax.'}}
        with patch('salt.auth.LoadAuth.authenticate_eauth', MagicMock(return_value=True)), \
             patch('salt.auth.LoadAuth.get_auth_list', MagicMock(return_value=['testing'])):
            ret = self.clear_funcs.runner(clear_load)

        self.assertDictEqual(mock_ret, ret)

    def test_runner_user_not_authenticated(self):
        '''
        Asserts that an UserAuthenticationError is returned when the user can't authenticate.
        '''
        mock_ret = {'error': {'name': 'UserAuthenticationError',
                              'message': 'Authentication failure of type "user" occurred'}}
        ret = self.clear_funcs.runner({})
        self.assertDictEqual(mock_ret, ret)

    # wheel tests

    def test_wheel_token_not_authenticated(self):
        '''
        Asserts that a TokenAuthenticationError is returned when the token can't authenticate.
        '''
        mock_ret = {'error': {'name': 'TokenAuthenticationError',
                              'message': 'Authentication failure of type "token" occurred.'}}
        ret = self.clear_funcs.wheel({'token': 'asdfasdfasdfasdf'})
        self.assertDictEqual(mock_ret, ret)

    def test_wheel_token_authorization_error(self):
        '''
        Asserts that a TokenAuthenticationError is returned when the token authenticates, but is
        not authorized.
        '''
        token = 'asdfasdfasdfasdf'
        clear_load = {'token': token, 'fun': 'test.arg'}
        mock_token = {'token': token, 'eauth': 'foo', 'name': 'test'}
        mock_ret = {'error': {'name': 'TokenAuthenticationError',
                              'message': 'Authentication failure of type "token" occurred '
                                         'for user test.'}}

        with patch('salt.auth.LoadAuth.authenticate_token', MagicMock(return_value=mock_token)), \
             patch('salt.auth.LoadAuth.get_auth_list', MagicMock(return_value=[])):
            ret = self.clear_funcs.wheel(clear_load)

        self.assertDictEqual(mock_ret, ret)

    def test_wheel_token_salt_invocation_error(self):
        '''
        Asserts that a SaltInvocationError is returned when the token authenticates, but the
        command is malformed.
        '''
        token = 'asdfasdfasdfasdf'
        clear_load = {'token': token, 'fun': 'badtestarg'}
        mock_token = {'token': token, 'eauth': 'foo', 'name': 'test'}
        mock_ret = {'error': {'name': 'SaltInvocationError',
                              'message': 'A command invocation error occurred: Check syntax.'}}

        with patch('salt.auth.LoadAuth.authenticate_token', MagicMock(return_value=mock_token)), \
             patch('salt.auth.LoadAuth.get_auth_list', MagicMock(return_value=['testing'])):
            ret = self.clear_funcs.wheel(clear_load)

        self.assertDictEqual(mock_ret, ret)

    def test_wheel_eauth_not_authenticated(self):
        '''
        Asserts that an EauthAuthenticationError is returned when the user can't authenticate.
        '''
        mock_ret = {'error': {'name': 'EauthAuthenticationError',
                              'message': 'Authentication failure of type "eauth" occurred for '
                                         'user UNKNOWN.'}}
        ret = self.clear_funcs.wheel({'eauth': 'foo'})
        self.assertDictEqual(mock_ret, ret)

    def test_wheel_eauth_authorization_error(self):
        '''
        Asserts that an EauthAuthenticationError is returned when the user authenticates, but is
        not authorized.
        '''
        clear_load = {'eauth': 'foo', 'username': 'test', 'fun': 'test.arg'}
        mock_ret = {'error': {'name': 'EauthAuthenticationError',
                              'message': 'Authentication failure of type "eauth" occurred for '
                                         'user test.'}}
        with patch('salt.auth.LoadAuth.authenticate_eauth', MagicMock(return_value=True)), \
             patch('salt.auth.LoadAuth.get_auth_list', MagicMock(return_value=[])):
            ret = self.clear_funcs.wheel(clear_load)

        self.assertDictEqual(mock_ret, ret)

    def test_wheel_eauth_salt_invocation_error(self):
        '''
        Asserts that an EauthAuthenticationError is returned when the user authenticates, but the
        command is malformed.
        '''
        clear_load = {'eauth': 'foo', 'username': 'test', 'fun': 'bad.test.arg.func'}
        mock_ret = {'error': {'name': 'SaltInvocationError',
                              'message': 'A command invocation error occurred: Check syntax.'}}
        with patch('salt.auth.LoadAuth.authenticate_eauth', MagicMock(return_value=True)), \
             patch('salt.auth.LoadAuth.get_auth_list', MagicMock(return_value=['testing'])):
            ret = self.clear_funcs.wheel(clear_load)

        self.assertDictEqual(mock_ret, ret)

    def test_wheel_user_not_authenticated(self):
        '''
        Asserts that an UserAuthenticationError is returned when the user can't authenticate.
        '''
        mock_ret = {'error': {'name': 'UserAuthenticationError',
                              'message': 'Authentication failure of type "user" occurred'}}
        ret = self.clear_funcs.wheel({})
        self.assertDictEqual(mock_ret, ret)

    # publish tests

    def test_publish_user_is_blacklisted(self):
        '''
        Asserts that an AuthorizationError is returned when the user has been blacklisted.
        '''
        mock_ret = {'error': {'name': 'AuthorizationError',
                              'message': 'Authorization error occurred.'}}
        with patch('salt.acl.PublisherACL.user_is_blacklisted', MagicMock(return_value=True)):
            self.assertEqual(mock_ret, self.clear_funcs.publish({'user': 'foo', 'fun': 'test.arg'}))

    def test_publish_cmd_blacklisted(self):
        '''
        Asserts that an AuthorizationError is returned when the command has been blacklisted.
        '''
        mock_ret = {'error': {'name': 'AuthorizationError',
                              'message': 'Authorization error occurred.'}}
        with patch('salt.acl.PublisherACL.user_is_blacklisted', MagicMock(return_value=False)), \
                patch('salt.acl.PublisherACL.cmd_is_blacklisted', MagicMock(return_value=True)):
            self.assertEqual(mock_ret, self.clear_funcs.publish({'user': 'foo', 'fun': 'test.arg'}))

    def test_publish_token_not_authenticated(self):
        '''
        Asserts that an AuthenticationError is returned when the token can't authenticate.
        '''
        mock_ret = {'error': {'name': 'AuthenticationError',
                              'message': 'Authentication error occurred.'}}
        load = {'user': 'foo', 'fun': 'test.arg', 'tgt': 'test_minion',
                'kwargs': {'token': 'asdfasdfasdfasdf'}}
        with patch('salt.acl.PublisherACL.user_is_blacklisted', MagicMock(return_value=False)), \
                patch('salt.acl.PublisherACL.cmd_is_blacklisted', MagicMock(return_value=False)):
            self.assertEqual(mock_ret, self.clear_funcs.publish(load))

    def test_publish_token_authorization_error(self):
        '''
        Asserts that an AuthorizationError is returned when the token authenticates, but is not
        authorized.
        '''
        token = 'asdfasdfasdfasdf'
        load = {'user': 'foo', 'fun': 'test.arg', 'tgt': 'test_minion',
                'arg': 'bar', 'kwargs': {'token': token}}
        mock_token = {'token': token, 'eauth': 'foo', 'name': 'test'}
        mock_ret = {'error': {'name': 'AuthorizationError',
                              'message': 'Authorization error occurred.'}}

        with patch('salt.acl.PublisherACL.user_is_blacklisted', MagicMock(return_value=False)), \
                patch('salt.acl.PublisherACL.cmd_is_blacklisted', MagicMock(return_value=False)), \
                patch('salt.auth.LoadAuth.authenticate_token', MagicMock(return_value=mock_token)), \
                patch('salt.auth.LoadAuth.get_auth_list', MagicMock(return_value=[])):
            self.assertEqual(mock_ret, self.clear_funcs.publish(load))

    def test_publish_eauth_not_authenticated(self):
        '''
        Asserts that an AuthenticationError is returned when the user can't authenticate.
        '''
        load = {'user': 'test', 'fun': 'test.arg', 'tgt': 'test_minion',
                'kwargs': {'eauth': 'foo'}}
        mock_ret = {'error': {'name': 'AuthenticationError',
                               'message': 'Authentication error occurred.'}}
        with patch('salt.acl.PublisherACL.user_is_blacklisted', MagicMock(return_value=False)), \
                patch('salt.acl.PublisherACL.cmd_is_blacklisted', MagicMock(return_value=False)):
            self.assertEqual(mock_ret, self.clear_funcs.publish(load))

    def test_publish_eauth_authorization_error(self):
        '''
        Asserts that an AuthorizationError is returned when the user authenticates, but is not
        authorized.
        '''
        load = {'user': 'test', 'fun': 'test.arg', 'tgt': 'test_minion',
                'kwargs': {'eauth': 'foo'}, 'arg': 'bar'}
        mock_ret = {'error': {'name': 'AuthorizationError',
                               'message': 'Authorization error occurred.'}}
        with patch('salt.acl.PublisherACL.user_is_blacklisted', MagicMock(return_value=False)), \
                patch('salt.acl.PublisherACL.cmd_is_blacklisted', MagicMock(return_value=False)), \
                patch('salt.auth.LoadAuth.authenticate_eauth', MagicMock(return_value=True)), \
                patch('salt.auth.LoadAuth.get_auth_list', MagicMock(return_value=[])):
            self.assertEqual(mock_ret, self.clear_funcs.publish(load))

    def test_publish_user_not_authenticated(self):
        '''
        Asserts that an AuthenticationError is returned when the user can't authenticate.
        '''
        load = {'user': 'test', 'fun': 'test.arg', 'tgt': 'test_minion'}
        mock_ret = {'error': {'name': 'AuthenticationError',
                              'message': 'Authentication error occurred.'}}
        with patch('salt.acl.PublisherACL.user_is_blacklisted', MagicMock(return_value=False)), \
                patch('salt.acl.PublisherACL.cmd_is_blacklisted', MagicMock(return_value=False)):
            self.assertEqual(mock_ret, self.clear_funcs.publish(load))

    def test_publish_user_authenticated_missing_auth_list(self):
        '''
        Asserts that an AuthenticationError is returned when the user has an effective user id and is
        authenticated, but the auth_list is empty.
        '''
        load = {'user': 'test', 'fun': 'test.arg', 'tgt': 'test_minion',
                'kwargs': {'user': 'test'}, 'arg': 'foo'}
        mock_ret = {'error': {'name': 'AuthenticationError',
                              'message': 'Authentication error occurred.'}}
        with patch('salt.acl.PublisherACL.user_is_blacklisted', MagicMock(return_value=False)), \
                patch('salt.acl.PublisherACL.cmd_is_blacklisted', MagicMock(return_value=False)), \
                patch('salt.auth.LoadAuth.authenticate_key', MagicMock(return_value='fake-user-key')), \
                patch('salt.utils.master.get_values_of_matching_keys', MagicMock(return_value=[])):
            self.assertEqual(mock_ret, self.clear_funcs.publish(load))

    def test_publish_user_authorization_error(self):
        '''
        Asserts that an AuthorizationError is returned when the user authenticates, but is not
        authorized.
        '''
        load = {'user': 'test', 'fun': 'test.arg', 'tgt': 'test_minion',
                'kwargs': {'user': 'test'}, 'arg': 'foo'}
        mock_ret = {'error': {'name': 'AuthorizationError',
                              'message': 'Authorization error occurred.'}}
        with patch('salt.acl.PublisherACL.user_is_blacklisted', MagicMock(return_value=False)), \
                patch('salt.acl.PublisherACL.cmd_is_blacklisted', MagicMock(return_value=False)), \
                patch('salt.auth.LoadAuth.authenticate_key', MagicMock(return_value='fake-user-key')), \
                patch('salt.utils.master.get_values_of_matching_keys', MagicMock(return_value=['test'])), \
                patch('salt.utils.minions.CkMinions.auth_check', MagicMock(return_value=False)):
            self.assertEqual(mock_ret, self.clear_funcs.publish(load))



# -*- coding: utf-8 -*-
'''
Common code used in Docker integration tests
'''
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import functools
import random
import string

# Import Salt libs
from salt.exceptions import CommandExecutionError

# Import 3rd-party libs
from salt._compat import ipaddress
from salt.ext.six.moves import range  # pylint: disable=import-error,redefined-builtin


def random_name(prefix=''):
    ret = prefix
    for _ in range(8):
        ret += random.choice(string.ascii_lowercase)
    return ret


class Network(object):
    def __init__(self, name, **kwargs):
        self.kwargs = kwargs
        self.name = name
        try:
            self.net = ipaddress.ip_network(self.kwargs['subnet'])
            self._rand_indexes = random.sample(
                range(2, self.net.num_addresses - 1),
                self.net.num_addresses - 3)
            self.ip_arg = 'ipv{0}_address'.format(self.net.version)
        except KeyError:
            # No explicit subnet passed
            self.net = self.ip_arg = None

    def __getitem__(self, index):
        try:
            return self.net[self._rand_indexes[index]].compressed
        except (TypeError, AttributeError):
            raise ValueError(
                'Indexing not supported for networks without a custom subnet')

    def arg_map(self, arg_name):
        return {'ipv4_address': 'IPv4Address',
                'ipv6_address': 'IPv6Address',
                'links': 'Links',
                'aliases': 'Aliases'}[arg_name]

    @property
    def subnet(self):
        try:
            return self.net.compressed
        except AttributeError:
            return None

    @property
    def gateway(self):
        try:
            return self.kwargs['gateway']
        except KeyError:
            try:
                return self.net[1].compressed
            except (AttributeError, IndexError):
                return None


class with_network(object):
    '''
    Generate a network for the test. Information about the network will be
    passed to the wrapped function.
    '''
    def __init__(self, **kwargs):
        self.create = kwargs.pop('create', False)
        self.network = Network(random_name(prefix='salt_net_'), **kwargs)
        if self.network.net is not None:
            if 'enable_ipv6' not in kwargs:
                kwargs['enable_ipv6'] = self.network.net.version == 6
        self.kwargs = kwargs

    def __call__(self, func):
        self.func = func
        return functools.wraps(func)(
            lambda testcase, *args, **kwargs: self.wrap(testcase, *args, **kwargs)  # pylint: disable=W0108
        )

    def wrap(self, testcase, *args, **kwargs):
        if self.create:
            testcase.run_function(
                'docker.create_network',
                [self.network.name],
                **self.kwargs)
        try:
            return self.func(testcase, self.network, *args, **kwargs)
        finally:
            try:
                testcase.run_function(
                    'docker.disconnect_all_containers_from_network',
                    [self.network.name])
            except CommandExecutionError as exc:
                if '404' not in exc.__str__():
                    raise
            else:
                testcase.run_function(
                    'docker.remove_network',
                    [self.network.name])



# -*- coding: utf-8 -*-
'''
integration tests for nilirt_ip
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import time

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.unit import skipIf
from tests.support.helpers import destructiveTest, skip_if_not_root

# Import Salt libs
import salt.utils.platform


@skip_if_not_root
@skipIf(not salt.utils.platform.is_linux(), 'These tests can only be run on linux')
class Nilrt_ipModuleTest(ModuleCase):
    '''
    Validate the nilrt_ip module
    '''

    def __init__(self, arg):
        super(self.__class__, self).__init__(arg)
        self.initialState = {}

    def setUp(self):
        '''
        Get current settings
        '''
        # save files from var/lib/connman*
        os_grain = self.run_function('grains.item', ['os_family'])
        if os_grain['os_family'] != 'NILinuxRT':
            self.skipTest('Tests applicable only to NILinuxRT')
        super(Nilrt_ipModuleTest, self).setUp()
        self.run_function('file.copy', ['/var/lib/connman', '/tmp/connman', 'recurse=True', 'remove_existing=True'])

    def tearDown(self):
        '''
        Reset to original settings
        '''
        # restore files
        self.run_function('file.copy', ['/tmp/connman', '/var/lib/connman', 'recurse=True', 'remove_existing=True'])
        # restart connman
        self.run_function('service.restart', ['connman'])
        time.sleep(10)  # wait 10 seconds for connman to be fully loaded
        interfaces = self.__interfaces()
        for interface in interfaces:
            self.run_function('ip.up', [interface])

    def __connected(self, interface):
        return interface['up']

    def __interfaces(self):
        interfaceList = []
        for iface in self.run_function('ip.get_interfaces_details')['interfaces']:
            interfaceList.append(iface['connectionid'])
        return interfaceList

    @destructiveTest
    def test_down(self):
        interfaces = self.__interfaces()
        for interface in interfaces:
            result = self.run_function('ip.down', [interface])
            self.assertTrue(result)
        info = self.run_function('ip.get_interfaces_details')
        for interface in info['interfaces']:
            self.assertFalse(self.__connected(interface))

    @destructiveTest
    def test_up(self):
        interfaces = self.__interfaces()
        #first down all interfaces
        for interface in interfaces:
            self.run_function('ip.down', [interface])
        # up interfaces
        for interface in interfaces:
            result = self.run_function('ip.up', [interface])
            self.assertTrue(result)
        info = self.run_function('ip.get_interfaces_details')
        for interface in info['interfaces']:
            self.assertTrue(self.__connected(interface))

    @destructiveTest
    def test_set_dhcp_linklocal_all(self):
        interfaces = self.__interfaces()
        for interface in interfaces:
            result = self.run_function('ip.set_dhcp_linklocal_all', [interface])
            self.assertTrue(result)
        info = self.run_function('ip.get_interfaces_details')
        for interface in info['interfaces']:
            self.assertEqual(interface['ipv4']['requestmode'], 'dhcp_linklocal')

    @destructiveTest
    def test_static_all(self):
        interfaces = self.__interfaces()
        for interface in interfaces:
            result = self.run_function('ip.set_static_all', [interface, '192.168.10.4', '255.255.255.0', '192.168.10.1', '8.8.4.4 8.8.8.8'])
            self.assertTrue(result)

        info = self.run_function('ip.get_interfaces_details')
        for interface in info['interfaces']:
            self.assertIn('8.8.4.4', interface['ipv4']['dns'])
            self.assertIn('8.8.8.8', interface['ipv4']['dns'])
            self.assertEqual(interface['ipv4']['requestmode'], 'static')
            self.assertEqual(interface['ipv4']['address'], '192.168.10.4')
            self.assertEqual(interface['ipv4']['netmask'], '255.255.255.0')
            self.assertEqual(interface['ipv4']['gateway'], '192.168.10.1')



# -*- coding: utf-8 -*-
'''
Tests for the salt runner

.. versionadded:: 2016.11.0
'''
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing libs
from tests.support.case import ShellCase


class SaltRunnerTest(ShellCase):
    '''
    Test the salt runner
    '''
    def test_salt_cmd(self):
        '''
        test return values of salt.cmd
        '''
        ret = self.run_run_plus('salt.cmd', 'test.ping')
        out_ret = ret.get('out')[0]
        return_ret = ret.get('return')

        self.assertEqual(out_ret, 'True')
        self.assertTrue(return_ret)



# -*- coding: utf-8 -*-

# Import Python Libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing Libs
from tests.support.helpers import destructiveTest
from tests.support.mock import NO_MOCK, NO_MOCK_REASON
from tests.support.unit import TestCase, skipIf

# Import Salt Libs
import salt.utils.platform
import salt.utils.win_lgpo_netsh as win_lgpo_netsh
from salt.exceptions import CommandExecutionError


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not salt.utils.platform.is_windows(), 'System is not Windows')
class WinLgpoNetshTestCase(TestCase):
    def test_get_settings_firewallpolicy_local(self):
        ret = win_lgpo_netsh.get_settings(profile='domain',
                                          section='firewallpolicy',
                                          store='local')
        self.assertIn('Inbound', ret)
        self.assertIn('Outbound', ret)

    def test_get_settings_firewallpolicy_lgpo(self):
        ret = win_lgpo_netsh.get_settings(profile='domain',
                                          section='firewallpolicy',
                                          store='lgpo')
        self.assertIn('Inbound', ret)
        self.assertIn('Outbound', ret)

    def test_get_settings_logging_local(self):
        ret = win_lgpo_netsh.get_settings(profile='domain',
                                          section='logging',
                                          store='local')
        self.assertIn('FileName', ret)
        self.assertIn('LogAllowedConnections', ret)
        self.assertIn('LogDroppedConnections', ret)
        self.assertIn('MaxFileSize', ret)

    def test_get_settings_logging_lgpo(self):
        ret = win_lgpo_netsh.get_settings(profile='domain',
                                          section='logging',
                                          store='lgpo')
        self.assertIn('FileName', ret)
        self.assertIn('LogAllowedConnections', ret)
        self.assertIn('LogDroppedConnections', ret)
        self.assertIn('MaxFileSize', ret)

    def test_get_settings_settings_local(self):
        ret = win_lgpo_netsh.get_settings(profile='domain',
                                          section='settings',
                                          store='local')
        self.assertIn('InboundUserNotification', ret)
        self.assertIn('LocalConSecRules', ret)
        self.assertIn('LocalFirewallRules', ret)
        self.assertIn('RemoteManagement', ret)
        self.assertIn('UnicastResponseToMulticast', ret)

    def test_get_settings_settings_lgpo(self):
        ret = win_lgpo_netsh.get_settings(profile='domain',
                                          section='settings',
                                          store='lgpo')
        self.assertIn('InboundUserNotification', ret)
        self.assertIn('LocalConSecRules', ret)
        self.assertIn('LocalFirewallRules', ret)
        self.assertIn('RemoteManagement', ret)
        self.assertIn('UnicastResponseToMulticast', ret)

    def test_get_settings_state_local(self):
        ret = win_lgpo_netsh.get_settings(profile='domain',
                                          section='state',
                                          store='local')
        self.assertIn('State', ret)

    def test_get_settings_state_lgpo(self):
        ret = win_lgpo_netsh.get_settings(profile='domain',
                                          section='state',
                                          store='lgpo')
        self.assertIn('State', ret)

    def test_get_all_settings_local(self):
        ret = win_lgpo_netsh.get_all_settings(profile='domain',
                                              store='local')

        self.assertIn('Inbound', ret)
        self.assertIn('Outbound', ret)
        self.assertIn('FileName', ret)
        self.assertIn('LogAllowedConnections', ret)
        self.assertIn('LogDroppedConnections', ret)
        self.assertIn('MaxFileSize', ret)
        self.assertIn('InboundUserNotification', ret)
        self.assertIn('LocalConSecRules', ret)
        self.assertIn('LocalFirewallRules', ret)
        self.assertIn('RemoteManagement', ret)
        self.assertIn('UnicastResponseToMulticast', ret)
        self.assertIn('State', ret)

    def test_get_all_settings_lgpo(self):
        ret = win_lgpo_netsh.get_all_settings(profile='domain',
                                              store='local')

        self.assertIn('Inbound', ret)
        self.assertIn('Outbound', ret)
        self.assertIn('FileName', ret)
        self.assertIn('LogAllowedConnections', ret)
        self.assertIn('LogDroppedConnections', ret)
        self.assertIn('MaxFileSize', ret)
        self.assertIn('InboundUserNotification', ret)
        self.assertIn('LocalConSecRules', ret)
        self.assertIn('LocalFirewallRules', ret)
        self.assertIn('RemoteManagement', ret)
        self.assertIn('UnicastResponseToMulticast', ret)
        self.assertIn('State', ret)

    def test_get_all_profiles_local(self):
        ret = win_lgpo_netsh.get_all_profiles(store='local')
        self.assertIn('Domain Profile', ret)
        self.assertIn('Private Profile', ret)
        self.assertIn('Public Profile', ret)

    def test_get_all_profiles_lgpo(self):
        ret = win_lgpo_netsh.get_all_profiles(store='lgpo')
        self.assertIn('Domain Profile', ret)
        self.assertIn('Private Profile', ret)
        self.assertIn('Public Profile', ret)

    @destructiveTest
    def test_set_firewall_settings_inbound_local(self):
        current = win_lgpo_netsh.get_settings(profile='domain',
                                              section='firewallpolicy',
                                              store='local')['Inbound']
        try:
            ret = win_lgpo_netsh.set_firewall_settings(profile='domain',
                                                       inbound='allowinbound',
                                                       store='local')
            self.assertTrue(ret)
            new = win_lgpo_netsh.get_settings(profile='domain',
                                                  section='firewallpolicy',
                                                  store='local')['Inbound']
            self.assertEqual('AllowInbound', new)
        finally:
            ret = win_lgpo_netsh.set_firewall_settings(profile='domain',
                                                       inbound=current,
                                                       store='local')
            self.assertTrue(ret)

    @destructiveTest
    def test_set_firewall_settings_inbound_local_notconfigured(self):
        current = win_lgpo_netsh.get_settings(profile='domain',
                                              section='firewallpolicy',
                                              store='local')['Inbound']
        try:
            self.assertRaises(
                CommandExecutionError,
                win_lgpo_netsh.set_firewall_settings,
                profile='domain',
                inbound='notconfigured',
                store='local')
        finally:
            ret = win_lgpo_netsh.set_firewall_settings(profile='domain',
                                                       inbound=current,
                                                       store='local')
            self.assertTrue(ret)

    @destructiveTest
    def test_set_firewall_settings_inbound_lgpo_notconfigured(self):
        current = win_lgpo_netsh.get_settings(profile='domain',
                                              section='firewallpolicy',
                                              store='lgpo')['Inbound']
        try:
            ret = win_lgpo_netsh.set_firewall_settings(profile='domain',
                                                       inbound='notconfigured',
                                                       store='lgpo')
            self.assertTrue(ret)
            new = win_lgpo_netsh.get_settings(profile='domain',
                                              section='firewallpolicy',
                                              store='lgpo')['Inbound']
            self.assertEqual('NotConfigured', new)
        finally:
            ret = win_lgpo_netsh.set_firewall_settings(profile='domain',
                                                       inbound=current,
                                                       store='lgpo')
            self.assertTrue(ret)

    @destructiveTest
    def test_set_firewall_settings_outbound_local(self):
        current = win_lgpo_netsh.get_settings(profile='domain',
                                              section='firewallpolicy',
                                              store='local')['Outbound']
        try:
            ret = win_lgpo_netsh.set_firewall_settings(profile='domain',
                                                       outbound='allowoutbound',
                                                       store='local')
            self.assertTrue(ret)
            new = win_lgpo_netsh.get_settings(profile='domain',
                                              section='firewallpolicy',
                                              store='local')['Outbound']
            self.assertEqual('AllowOutbound', new)
        finally:
            ret = win_lgpo_netsh.set_firewall_settings(profile='domain',
                                                       outbound=current,
                                                       store='local')
            self.assertTrue(ret)

    @destructiveTest
    def test_set_firewall_logging_allowed_local_enable(self):
        current = win_lgpo_netsh.get_settings(profile='domain',
                                              section='logging',
                                              store='local')['LogAllowedConnections']
        try:
            ret = win_lgpo_netsh.set_logging_settings(profile='domain',
                                                      setting='allowedconnections',
                                                      value='enable',
                                                      store='local')
            self.assertTrue(ret)
            new = win_lgpo_netsh.get_settings(profile='domain',
                                              section='logging',
                                              store='local')['LogAllowedConnections']
            self.assertEqual('Enable', new)
        finally:
            ret = win_lgpo_netsh.set_logging_settings(profile='domain',
                                                      setting='allowedconnections',
                                                      value=current,
                                                      store='local')
            self.assertTrue(ret)

    @destructiveTest
    def test_set_firewall_logging_allowed_local_notconfigured(self):
        current = win_lgpo_netsh.get_settings(profile='domain',
                                              section='logging',
                                              store='local')['LogAllowedConnections']
        try:
            self.assertRaises(
                CommandExecutionError,
                win_lgpo_netsh.set_logging_settings,
                profile='domain',
                setting='allowedconnections',
                value='notconfigured',
                store='local')
        finally:
            ret = win_lgpo_netsh.set_logging_settings(profile='domain',
                                                      setting='allowedconnections',
                                                      value=current,
                                                      store='local')
            self.assertTrue(ret)

    @destructiveTest
    def test_set_firewall_logging_allowed_lgpo_notconfigured(self):
        current = win_lgpo_netsh.get_settings(profile='domain',
                                              section='logging',
                                              store='lgpo')['LogAllowedConnections']
        try:
            ret = win_lgpo_netsh.set_logging_settings(profile='domain',
                                                      setting='allowedconnections',
                                                      value='notconfigured',
                                                      store='lgpo')
            self.assertTrue(ret)
            new = win_lgpo_netsh.get_settings(profile='domain',
                                              section='logging',
                                              store='lgpo')['LogAllowedConnections']
            self.assertEqual('NotConfigured', new)
        finally:
            ret = win_lgpo_netsh.set_logging_settings(profile='domain',
                                                      setting='allowedconnections',
                                                      value=current,
                                                      store='lgpo')
            self.assertTrue(ret)

    def test_set_firewall_logging_dropped_local_enable(self):
        current = win_lgpo_netsh.get_settings(profile='domain',
                                              section='logging',
                                              store='local')['LogDroppedConnections']
        try:
            ret = win_lgpo_netsh.set_logging_settings(profile='domain',
                                                      setting='droppedconnections',
                                                      value='enable',
                                                      store='local')
            self.assertTrue(ret)
            new = win_lgpo_netsh.get_settings(profile='domain',
                                              section='logging',
                                              store='local')['LogDroppedConnections']
            self.assertEqual('Enable', new)
        finally:
            ret = win_lgpo_netsh.set_logging_settings(profile='domain',
                                                      setting='droppedconnections',
                                                      value=current,
                                                      store='local')
            self.assertTrue(ret)

    def test_set_firewall_logging_filename_local(self):
        current = win_lgpo_netsh.get_settings(profile='domain',
                                              section='logging',
                                              store='local')['FileName']
        try:
            ret = win_lgpo_netsh.set_logging_settings(profile='domain',
                                                      setting='filename',
                                                      value='C:\\Temp\\test.log',
                                                      store='local')
            self.assertTrue(ret)
            new = win_lgpo_netsh.get_settings(profile='domain',
                                              section='logging',
                                              store='local')['FileName']
            self.assertEqual('C:\\Temp\\test.log', new)
        finally:
            ret = win_lgpo_netsh.set_logging_settings(profile='domain',
                                                      setting='filename',
                                                      value=current,
                                                      store='local')
            self.assertTrue(ret)

    def test_set_firewall_logging_maxfilesize_local(self):
        current = win_lgpo_netsh.get_settings(profile='domain',
                                              section='logging',
                                              store='local')['MaxFileSize']
        try:
            ret = win_lgpo_netsh.set_logging_settings(profile='domain',
                                                      setting='maxfilesize',
                                                      value='16384',
                                                      store='local')
            self.assertTrue(ret)
            new = win_lgpo_netsh.get_settings(profile='domain',
                                              section='logging',
                                              store='local')['MaxFileSize']
            self.assertEqual('16384', new)
        finally:
            ret = win_lgpo_netsh.set_logging_settings(profile='domain',
                                                      setting='maxfilesize',
                                                      value=current,
                                                      store='local')
            self.assertTrue(ret)

    @destructiveTest
    def test_set_firewall_settings_fwrules_local_enable(self):
        self.assertRaises(
            CommandExecutionError,
            win_lgpo_netsh.set_settings,
            profile='domain',
            setting='localfirewallrules',
            value='enable',
            store='local')

    @destructiveTest
    def test_set_firewall_settings_fwrules_lgpo_notconfigured(self):
        current = win_lgpo_netsh.get_settings(profile='domain',
                                              section='settings',
                                              store='lgpo')['LocalFirewallRules']
        try:
            ret = win_lgpo_netsh.set_settings(profile='domain',
                                              setting='localfirewallrules',
                                              value='notconfigured',
                                              store='lgpo')
            self.assertTrue(ret)
            new = win_lgpo_netsh.get_settings(profile='domain',
                                              section='settings',
                                              store='lgpo')['LocalFirewallRules']
            self.assertEqual('NotConfigured', new)
        finally:
            ret = win_lgpo_netsh.set_settings(profile='domain',
                                              setting='localfirewallrules',
                                              value=current,
                                              store='lgpo')
            self.assertTrue(ret)

    @destructiveTest
    def test_set_firewall_settings_consecrules_local_enable(self):
        self.assertRaises(
            CommandExecutionError,
            win_lgpo_netsh.set_settings,
            profile='domain',
            setting='localconsecrules',
            value='enable',
            store='local')

    def test_set_firewall_settings_notification_local_enable(self):
        current = win_lgpo_netsh.get_settings(profile='domain',
                                              section='settings',
                                              store='local')['InboundUserNotification']
        try:
            ret = win_lgpo_netsh.set_settings(profile='domain',
                                              setting='inboundusernotification',
                                              value='enable',
                                              store='local')
            self.assertTrue(ret)
            new = win_lgpo_netsh.get_settings(profile='domain',
                                              section='settings',
                                              store='local')['InboundUserNotification']
            self.assertEqual('Enable', new)
        finally:
            ret = win_lgpo_netsh.set_settings(profile='domain',
                                              setting='inboundusernotification',
                                              value=current,
                                              store='local')
            self.assertTrue(ret)

    @destructiveTest
    def test_set_firewall_settings_notification_local_notconfigured(self):
        current = win_lgpo_netsh.get_settings(profile='domain',
                                              section='settings',
                                              store='local')['InboundUserNotification']
        try:
            self.assertRaises(
                CommandExecutionError,
                win_lgpo_netsh.set_settings,
                profile='domain',
                setting='inboundusernotification',
                value='notconfigured',
                store='local')
        finally:
            ret = win_lgpo_netsh.set_settings(profile='domain',
                                              setting='inboundusernotification',
                                              value=current,
                                              store='local')
            self.assertTrue(ret)

    def test_set_firewall_settings_notification_lgpo_notconfigured(self):
        current = win_lgpo_netsh.get_settings(profile='domain',
                                              section='settings',
                                              store='lgpo')['InboundUserNotification']
        try:
            ret = win_lgpo_netsh.set_settings(profile='domain',
                                              setting='inboundusernotification',
                                              value='notconfigured',
                                              store='lgpo')
            self.assertTrue(ret)
            new = win_lgpo_netsh.get_settings(profile='domain',
                                              section='settings',
                                              store='lgpo')['InboundUserNotification']
            self.assertEqual('NotConfigured', new)
        finally:
            ret = win_lgpo_netsh.set_settings(profile='domain',
                                              setting='inboundusernotification',
                                              value=current,
                                              store='lgpo')
            self.assertTrue(ret)

    def test_set_firewall_settings_remotemgmt_local_enable(self):
        current = win_lgpo_netsh.get_settings(profile='domain',
                                              section='settings',
                                              store='local')['RemoteManagement']
        try:
            ret = win_lgpo_netsh.set_settings(profile='domain',
                                              setting='remotemanagement',
                                              value='enable',
                                              store='local')
            self.assertTrue(ret)
            new = win_lgpo_netsh.get_settings(profile='domain',
                                              section='settings',
                                              store='local')['RemoteManagement']
            self.assertEqual('Enable', new)
        finally:
            ret = win_lgpo_netsh.set_settings(profile='domain',
                                              setting='remotemanagement',
                                              value=current,
                                              store='local')
            self.assertTrue(ret)

    def test_set_firewall_settings_unicast_local_disable(self):
        current = win_lgpo_netsh.get_settings(profile='domain',
                                              section='settings',
                                              store='local')['UnicastResponseToMulticast']
        try:
            ret = win_lgpo_netsh.set_settings(profile='domain',
                                              setting='unicastresponsetomulticast',
                                              value='disable',
                                              store='local')
            self.assertTrue(ret)
            new = win_lgpo_netsh.get_settings(profile='domain',
                                              section='settings',
                                              store='local')['UnicastResponseToMulticast']
            self.assertEqual('Disable', new)
        finally:
            ret = win_lgpo_netsh.set_settings(profile='domain',
                                              setting='unicastresponsetomulticast',
                                              value=current,
                                              store='local')
            self.assertTrue(ret)

    @destructiveTest
    def test_set_firewall_state_local_on(self):
        current = win_lgpo_netsh.get_settings(profile='domain',
                                              section='state',
                                              store='local')['State']
        try:
            ret = win_lgpo_netsh.set_state(profile='domain',
                                           state='off',
                                           store='local')
            self.assertTrue(ret)
            new = win_lgpo_netsh.get_settings(profile='domain',
                                              section='state',
                                              store='local')['State']
            self.assertEqual('OFF', new)
        finally:
            ret = win_lgpo_netsh.set_state(profile='domain',
                                           state=current,
                                           store='local')
            self.assertTrue(ret)

    @destructiveTest
    def test_set_firewall_state_local_notconfigured(self):
        current = win_lgpo_netsh.get_settings(profile='domain',
                                              section='state',
                                              store='local')['State']
        try:
            self.assertRaises(
                CommandExecutionError,
                win_lgpo_netsh.set_state,
                profile='domain',
                state='notconfigured',
                store='local')
        finally:
            ret = win_lgpo_netsh.set_state(profile='domain',
                                           state=current,
                                           store='local')
            self.assertTrue(ret)

    @destructiveTest
    def test_set_firewall_state_lgpo_notconfigured(self):
        current = win_lgpo_netsh.get_settings(profile='domain',
                                              section='state',
                                              store='local')['State']
        try:
            ret = win_lgpo_netsh.set_state(profile='domain',
                                           state='notconfigured',
                                           store='lgpo')
            self.assertTrue(ret)
            new = win_lgpo_netsh.get_settings(profile='domain',
                                              section='state',
                                              store='lgpo')['State']
            self.assertEqual('NotConfigured', new)
        finally:
            ret = win_lgpo_netsh.set_state(profile='domain',
                                           state=current,
                                           store='lgpo')
            self.assertTrue(ret)



# -*- coding: utf-8 -*-
'''
Tests for the salt-run command
'''
# Import Python libs
from __future__ import absolute_import
import functools
import random
import string

# Import Salt Testing libs
from tests.support.case import ShellCase
from tests.support.helpers import destructiveTest, expensiveTest
from salt.ext.six.moves import range


def _random_name(prefix=''):
    ret = prefix
    for _ in range(8):
        ret += random.choice(string.ascii_lowercase)
    return ret


def with_random_name(func):
    '''
    generate a randomized name for a container
    '''
    @functools.wraps(func)
    def wrapper(self, *args, **kwargs):
        name = _random_name(prefix='salt_')
        return func(self, _random_name(prefix='salt_test_'), *args, **kwargs)
    return wrapper


@destructiveTest
@expensiveTest
class VenafiTest(ShellCase):
    '''
    Test the venafi runner
    '''

    @with_random_name
    def test_gen_key_password(self, name):
        '''
        venafi.gen_key
        '''
        ret = self.run_run_plus(fun='venafi.gen_key',
                                minion_id='{0}.test.saltstack.com'.format(name),
                                dns_name='{0}.test.saltstack.com'.format(name),
                                zone='Internet',
                                password='SecretSauce')
        self.assertEqual(ret['out'][0], '-----BEGIN RSA PRIVATE KEY-----')
        self.assertEqual(ret['out'][1], 'Proc-Type: 4,ENCRYPTED')
        self.assertEqual(ret['out'][-1], '-----END RSA PRIVATE KEY-----')

    @with_random_name
    def test_gen_key_without_password(self, name):
        '''
        venafi.gen_key
        '''
        ret = self.run_run_plus(fun='venafi.gen_key',
                                minion_id='{0}.test.saltstack.com'.format(name),
                                dns_name='{0}.test.saltstack.com'.format(name),
                                zone='Internet')
        self.assertEqual(ret['out'][0], '-----BEGIN RSA PRIVATE KEY-----')
        self.assertNotEqual(ret['out'][1], 'Proc-Type: 4,ENCRYPTED')
        self.assertEqual(ret['out'][-1], '-----END RSA PRIVATE KEY-----')

    @with_random_name
    def test_gen_csr(self, name):
        '''
        venafi.gen_csr
        '''
        ret = self.run_run_plus(fun='venafi.gen_csr',
                                minion_id='{0}.test.saltstack.com'.format(name),
                                dns_name='{0}.test.saltstack.com'.format(name),
                                country='US', state='Utah', loc='Salt Lake City',
                                org='Salt Stack Inc.', org_unit='Testing',
                                zone='Internet', password='SecretSauce')
        self.assertEqual(ret['out'][0], '-----BEGIN CERTIFICATE REQUEST-----')
        self.assertEqual(ret['out'][-1], '-----END CERTIFICATE REQUEST-----')

    @with_random_name
    def test_request(self, name):
        '''
        venafi.request
        '''
        ret = self.run_run_plus(fun='venafi.request',
                                minion_id='{0}.example.com'.format(name),
                                dns_name='{0}.example.com'.format(name),
                                country='US', state='Utah', loc='Salt Lake City',
                                org='Salt Stack Inc.', org_unit='Testing',
                                zone='Internet', password='SecretSauce')
        self.assertTrue('request_id' in ret['return'])



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.transport.client

# Import 3rd-party libs
from salt.ext import six


class ReqChannelMixin(object):
    def test_basic(self):
        '''
        Test a variety of messages, make sure we get the expected responses
        '''
        msgs = [
            {'foo': 'bar'},
            {'bar': 'baz'},
            {'baz': 'qux', 'list': [1, 2, 3]},
        ]
        for msg in msgs:
            ret = self.channel.send(msg, timeout=2, tries=1)
            self.assertEqual(ret['load'], msg)

    def test_normalization(self):
        '''
        Since we use msgpack, we need to test that list types are converted to lists
        '''
        types = {
            'list': list,
        }
        msgs = [
            {'list': tuple([1, 2, 3])},
        ]
        for msg in msgs:
            ret = self.channel.send(msg, timeout=2, tries=1)
            for k, v in six.iteritems(ret['load']):
                self.assertEqual(types[k], type(v))

    def test_badload(self):
        '''
        Test a variety of bad requests, make sure that we get some sort of error
        '''
        msgs = ['', [], tuple()]
        for msg in msgs:
            ret = self.channel.send(msg, timeout=2, tries=1)
            self.assertEqual(ret, 'payload and load must be a dict')


class PubChannelMixin(object):
    def test_basic(self):
        self.pub = None

        def handle_pub(ret):
            self.pub = ret
            self.stop()
        self.pub_channel = salt.transport.client.AsyncPubChannel.factory(self.minion_opts, io_loop=self.io_loop)
        connect_future = self.pub_channel.connect()
        connect_future.add_done_callback(lambda f: self.stop())
        self.wait()
        connect_future.result()
        self.pub_channel.on_recv(handle_pub)
        load = {
            'fun': 'f',
            'arg': 'a',
            'tgt': 't',
            'jid': 'j',
            'ret': 'r',
            'tgt_type': 'glob',
        }
        self.server_channel.publish(load)
        self.wait()
        self.assertEqual(self.pub['load'], load)
        self.pub_channel.on_recv(None)
        self.server_channel.publish(load)
        with self.assertRaises(self.failureException):
            self.wait(timeout=0.5)

        # close our pub_channel, to pass our FD checks
        del self.pub_channel



# -*- coding: utf-8 -*-
'''test for pillar hg_pillar.py'''

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

import os
import tempfile
import shutil
import subprocess

# Import Salt Testing libs
from tests.integration import AdaptedConfigurationTestCaseMixin
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import NO_MOCK, NO_MOCK_REASON
from tests.support.paths import TMP


COMMIT_USER_NAME = 'test_user'
# file contents
PILLAR_CONTENT = {'gna': 'hello'}
FILE_DATA = {
    'top.sls': {'base': {'*': ['user']}},
    'user.sls': PILLAR_CONTENT
}

# Import Salt Libs
import salt.utils.files
import salt.utils.yaml
import salt.pillar.hg_pillar as hg_pillar
HGLIB = hg_pillar.hglib


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(HGLIB is None, 'python-hglib library not installed')
class HgPillarTestCase(TestCase, AdaptedConfigurationTestCaseMixin, LoaderModuleMockMixin):
    'test hg_pillar pillar'
    maxDiff = None

    def setup_loader_modules(self):
        self.tmpdir = tempfile.mkdtemp(dir=TMP)
        self.addCleanup(shutil.rmtree, self.tmpdir)
        cachedir = os.path.join(self.tmpdir, 'cachedir')
        os.makedirs(os.path.join(cachedir, 'hg_pillar'))
        self.hg_repo_path = self._create_hg_repo()
        return {
            hg_pillar: {
                '__opts__':  {
                    'cachedir': cachedir,
                    'pillar_roots': {},
                    'file_roots': {},
                    'state_top': 'top.sls',
                    'extension_modules': '',
                    'renderer': 'yaml_jinja',
                    'pillar_opts': False,
                    'renderer_blacklist': [],
                    'renderer_whitelist': [],
                    'file_ignore_glob': [],
                    'file_ignore_regex': [],
                }
            }
        }

    def _create_hg_repo(self):
        'create repo in tempdir'
        hg_repo = os.path.join(self.tmpdir, 'repo_pillar')
        os.makedirs(hg_repo)
        subprocess.check_call(['hg', 'init', hg_repo])
        for filename in FILE_DATA:
            with salt.utils.files.fopen(os.path.join(hg_repo, filename), 'w') as data_file:
                salt.utils.yaml.safe_dump(FILE_DATA[filename], data_file)
        subprocess.check_call(['hg', 'ci', '-A', '-R', hg_repo, '-m', 'first commit', '-u', COMMIT_USER_NAME])
        return hg_repo

    def test_base(self):
        'check hg repo is imported correctly'
        mypillar = hg_pillar.ext_pillar('*', None, 'file://{0}'.format(self.hg_repo_path))
        self.assertEqual(PILLAR_CONTENT, mypillar)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
'''
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch,
    mock_open,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
import salt.modules.data as data


@skipIf(NO_MOCK, NO_MOCK_REASON)
class DataTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.data
    '''
    def setup_loader_modules(self):
        return {data: {}}

    # 'clear' function tests: 1

    def test_clear(self):
        '''
        Test if it clear out all of the data in the minion datastore
        '''
        with patch('os.remove', MagicMock(return_value='')):
            with patch.dict(data.__opts__, {'cachedir': ''}):
                self.assertTrue(data.clear())

    # 'load' function tests: 1

    def test_load(self):
        '''
        Test if it return all of the data in the minion datastore
        '''
        with patch('salt.payload.Serial.load', MagicMock(return_value=True)):
            mocked_fopen = MagicMock(return_value=True)
            mocked_fopen.__enter__ = MagicMock(return_value=mocked_fopen)
            mocked_fopen.__exit__ = MagicMock()
            with patch('salt.utils.files.fopen', MagicMock(return_value=mocked_fopen)):
                with patch('salt.payload.Serial.loads', MagicMock(return_value=True)):
                    with patch.dict(data.__opts__, {'cachedir': '/'}):
                        self.assertTrue(data.load())

    # 'dump' function tests: 3

    def test_dump(self):
        '''
        Test if it replace the entire datastore with a passed data structure
        '''
        with patch.dict(data.__opts__, {'cachedir': '/'}):
            with patch('salt.utils.files.fopen', mock_open()):
                self.assertTrue(data.dump('{"eggs": "spam"}'))

    def test_dump_isinstance(self):
        '''
        Test if it replace the entire datastore with a passed data structure
        '''
        with patch('ast.literal_eval', MagicMock(return_value='')):
            self.assertFalse(data.dump('salt'))

    def test_dump_ioerror(self):
        '''
        Test if it replace the entire datastore with a passed data structure
        '''
        with patch.dict(data.__opts__, {'cachedir': '/'}):
            mock = MagicMock(side_effect=IOError(''))
            with patch('salt.utils.files.fopen', mock):
                self.assertFalse(data.dump('{"eggs": "spam"}'))

    # 'update' function tests: 1

    def test_update(self):
        '''
        Test if it update a key with a value in the minion datastore
        '''
        with patch('salt.modules.data.load', MagicMock(return_value={})), \
                patch('salt.modules.data.dump', MagicMock(return_value=True)):
            self.assertTrue(data.update('foo', 'salt'))

    # 'get' function tests: 2

    def test_get(self):
        '''
        Test if it gets a value from the minion datastore
        '''
        with patch('salt.modules.data.load', MagicMock(return_value={'salt': 'SALT'})):
            self.assertEqual(data.get('salt'), 'SALT')

    def test_get_vals(self):
        '''
        Test if it gets values from the minion datastore
        '''
        with patch('salt.modules.data.load',
                   MagicMock(return_value={'salt': 'SALT', 'salt1': 'SALT1'})):
            self.assertEqual(data.get(['salt', 'salt1']), ['SALT', 'SALT1'])

    # 'cas' function tests: 1

    def test_cas_not_load(self):
        '''
        Test if it check and set a value in the minion datastore
        '''
        with patch('salt.modules.data.load',
                   MagicMock(return_value={'salt': 'SALT', 'salt1': 'SALT1'})):
            self.assertFalse(data.cas('salt3', 'SALT', 'SALTSTACK'))

    def test_cas_not_equal(self):
        '''
        Test if it check and set a value in the minion datastore
        '''
        with patch('salt.modules.data.load',
                   MagicMock(return_value={'salt': 'SALT', 'salt1': 'SALT1'})):
            self.assertFalse(data.cas('salt', 'SALT', 'SALTSTACK'))

    def test_cas(self):
        '''
        Test if it check and set a value in the minion datastore
        '''
        with patch('salt.modules.data.load',
                   MagicMock(return_value={'salt': 'SALT', 'salt1': 'SALT1'})), \
                           patch('salt.modules.data.dump',
                                 MagicMock(return_value=True)):
            self.assertTrue(data.cas('salt', 'SALTSTACK', 'SALT'))



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, MagicMock

# Import Salt Libs
from salt.pillar import extra_minion_data_in_pillar


@skipIf(NO_MOCK, NO_MOCK_REASON)
class ExtraMinionDataInPillarTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.pillar.extra_minion_data_in_pillar
    '''
    def setup_loader_modules(self):
        return {
            extra_minion_data_in_pillar: {
                '__virtual__': True,
            }
        }

    def setUp(self):
        self.pillar = MagicMock()
        self.extra_minion_data = {'key1': {'subkey1': 'value1'},
                                  'key2': {'subkey2': {'subsubkey2': 'value2'}},
                                  'key3': 'value3',
                                  'key4': {'subkey4': 'value4'}}

    def test_extra_values_none_or_empty(self):
        ret = extra_minion_data_in_pillar.ext_pillar('fake_id', self.pillar,
                                                     'fake_include', None)
        self.assertEqual(ret, {})
        ret = extra_minion_data_in_pillar.ext_pillar('fake_id', self.pillar,
                                                     'fake_include', {})
        self.assertEqual(ret, {})

    def test_include_all(self):
        for include_all in ['*', '<all>']:
            ret = extra_minion_data_in_pillar.ext_pillar(
                'fake_id', self.pillar, include_all, self.extra_minion_data)
            self.assertEqual(ret, self.extra_minion_data)

    def test_include_specific_keys(self):
        # Tests partially existing key, key with and without subkey,
        ret = extra_minion_data_in_pillar.ext_pillar(
            'fake_id', self.pillar,
            include=['key1:subkey1', 'key2:subkey3', 'key3', 'key4'],
            extra_minion_data=self.extra_minion_data)
        self.assertEqual(ret, {'key1': {'subkey1': 'value1'},
                               'key3': 'value3',
                               'key4': {'subkey4': 'value4'}})



# -*- coding: utf-8 -*-
#
# Copyright 2016 SUSE LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

'''
    :codeauthor: Bo Maryniuk <bo@suse.de>
'''

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals
import io

# Import Salt Testing Libs
from tests.support.unit import TestCase
from tests.support.mock import MagicMock, patch

from salt.modules.inspectlib.fsdb import CsvDB
from salt.modules.inspectlib.entities import CsvDBEntity
from salt.utils.odict import OrderedDict

from salt.ext import six
from salt.ext.six.moves import StringIO


def mock_open(data=None):
    '''
    Mock "open" function in a simple way.

    :param data:
    :return:
    '''
    data = StringIO(data)
    mock = MagicMock(spec=io.FileIO)
    handle = MagicMock(spec=io.FileIO)
    handle.write.return_value = None
    handle.__enter__.return_value = data or handle
    mock.return_value = handle

    return mock


class Writable(StringIO):
    def __init__(self, data=None):
        if data:
            StringIO.__init__(self, data)
        else:
            StringIO.__init__(self)
        self.data = []

    def __exit__(self, exc_type, exc_val, exc_tb):
        return self

    def __enter__(self):
        return self

    def write(self, s):
        self.data.append(s)


class FoobarEntity(CsvDBEntity):
    '''
    Entity for test purposes.
    '''
    _TABLE = 'some_table'

    def __init__(self):
        self.foo = 0
        self.bar = ''
        self.spam = 0.


class InspectorFSDBTestCase(TestCase):
    '''
    Test case for the FSDB: FileSystem Database.

    FSDB is a very simple object-to-CSV storage with a very inefficient
    update/delete operations (nice to have at some point) and efficient
    storing/reading the objects (what is exactly needed for the functionality).

    Main advantage of FSDB is to store Python objects in just a CSV files,
    and have a very small code base.
    '''

    def setUp(self):
        patcher = patch("os.makedirs", MagicMock())
        patcher.start()
        self.addCleanup(patcher.stop)

    def test_open(self):
        '''
        Test opening the database.
        :return:
        '''
        with patch("os.listdir", MagicMock(return_value=['test_db'])), \
                patch("gzip.open", mock_open("foo:int,bar:str")):
            csvdb = CsvDB('/foobar')
            csvdb.open()
            assert list(csvdb.list_tables()) == ['test_db']
            assert csvdb.is_closed() is False

    def test_close(self):
        '''
        Test closing the database.
        :return:
        '''
        with patch("os.listdir", MagicMock(return_value=['test_db'])), \
                patch("gzip.open", mock_open("foo:int,bar:str")):
            csvdb = CsvDB('/foobar')
            csvdb.open()
            csvdb.close()
            assert csvdb.is_closed() is True

    def test_create_table(self):
        '''
        Test creating table.
        :return:
        '''
        with patch("os.path.exists", MagicMock(return_value=False)), \
                patch("os.listdir", MagicMock(return_value=['some_table'])):
            writable = Writable()
            with patch("gzip.open", MagicMock(return_value=writable)):
                csvdb = CsvDB('/foobar')
                csvdb.open()
                csvdb.create_table_from_object(FoobarEntity())

            if six.PY2:
                assert writable.data[0].strip() == "foo:int,bar:unicode,spam:float"
            else:
                # Order in PY3 is not the same for every run
                writable_data = writable.data[0].strip()
                assert_order_options = ['bar:str,foo:int,spam:float',
                                        'bar:str,spam:float,foo:int',
                                        'foo:int,spam:float,bar:str',
                                        'foo:int,bar:str,spam:float',
                                        'spam:float,foo:int,bar:str',
                                        'spam:float,bar:str,foo:int']
                while assert_order_options:
                    assert_option = assert_order_options.pop()
                    try:
                        assert writable_data == assert_option
                        break
                    except AssertionError:
                        if not assert_order_options:
                            raise
                        continue

    def test_list_databases(self):
        '''
        Test list databases.
        :return:
        '''
        with patch("os.listdir", MagicMock(return_value=['test_db'])):
            csvdb = CsvDB('/foobar')
            assert csvdb.list() == ['test_db']

    def test_add_object(self):
        '''
        Test storing object into the database.
        :return:
        '''
        with patch("os.path.exists", MagicMock(return_value=False)), \
                patch("os.listdir", MagicMock(return_value=['some_table'])):
            writable = Writable()
            with patch("gzip.open", MagicMock(return_value=writable)):
                obj = FoobarEntity()
                obj.foo = 123
                obj.bar = 'test entity'
                obj.spam = 0.123

                csvdb = CsvDB('/foobar')
                csvdb.open()
                csvdb._tables = {'some_table': OrderedDict([tuple(elm.split(':'))
                                                            for elm in ["foo:int", "bar:str", "spam:float"]])}
                csvdb.store(obj)
                assert writable.data[0].strip() == '123,test entity,0.123'

    def test_delete_object(self):
        '''
        Deleting an object from the store.
        :return:
        '''
        with patch("gzip.open", MagicMock()), \
                patch("os.listdir", MagicMock(return_value=['test_db'])), \
                patch("csv.reader", MagicMock(return_value=iter([[], ['foo:int', 'bar:str', 'spam:float'],
                                                                ['123', 'test', '0.123'],
                                                                ['234', 'another', '0.456']]))):
            class InterceptedCsvDB(CsvDB):
                def __init__(self, path):
                    CsvDB.__init__(self, path)
                    self._remained = list()

                def store(self, obj, distinct=False):
                    self._remained.append(obj)

            csvdb = InterceptedCsvDB('/foobar')
            csvdb.open()
            csvdb.create_table_from_object = MagicMock()
            csvdb.flush = MagicMock()

            assert csvdb.delete(FoobarEntity, eq={'foo': 123}) is True
            assert len(csvdb._remained) == 1

            assert csvdb._remained[0].foo == 234
            assert csvdb._remained[0].bar == 'another'
            assert csvdb._remained[0].spam == 0.456

    def test_update_object(self):
        '''
        Updating an object from the store.
        :return:
        '''
        with patch("gzip.open", MagicMock()), \
                patch("os.listdir", MagicMock(return_value=['test_db'])), \
                patch("csv.reader", MagicMock(return_value=iter([[], ['foo:int', 'bar:str', 'spam:float'],
                                                                ['123', 'test', '0.123'],
                                                                ['234', 'another', '0.456']]))):
            obj = FoobarEntity()
            obj.foo = 123
            obj.bar = 'updated'
            obj.spam = 0.5

            class InterceptedCsvDB(CsvDB):
                def __init__(self, path):
                    CsvDB.__init__(self, path)
                    self._remained = list()

                def store(self, obj, distinct=False):
                    self._remained.append(obj)

            csvdb = InterceptedCsvDB('/foobar')
            csvdb.open()
            csvdb.create_table_from_object = MagicMock()
            csvdb.flush = MagicMock()

            assert csvdb.update(obj, eq={'foo': 123}) is True
            assert len(csvdb._remained) == 2

            assert csvdb._remained[0].foo == 123
            assert csvdb._remained[0].bar == 'updated'
            assert csvdb._remained[0].spam == 0.5

            assert csvdb._remained[1].foo == 234
            assert csvdb._remained[1].bar == 'another'
            assert csvdb._remained[1].spam == 0.456

    def test_get_object(self):
        '''
        Getting an object from the store.
        :return:
        '''
        with patch("os.listdir", MagicMock(return_value=['test_db'])), \
                patch("gzip.open", MagicMock()), \
                patch("csv.reader", MagicMock(return_value=iter([[], ['foo:int', 'bar:str', 'spam:float'],
                                                                 ['123', 'test', '0.123'],
                                                                 ['234', 'another', '0.456']]))):
            csvdb = CsvDB('/foobar')
            csvdb.open()
            entities = csvdb.get(FoobarEntity)
            assert list == type(entities)
            assert len(entities) == 2

            assert entities[0].foo == 123
            assert entities[0].bar == 'test'
            assert entities[0].spam == 0.123

            assert entities[1].foo == 234
            assert entities[1].bar == 'another'
            assert entities[1].spam == 0.456

    def test_get_obj_equals(self):
        '''
        Getting an object from the store with conditions
        :return:
        '''
        with patch("os.listdir", MagicMock(return_value=['test_db'])), \
            patch("gzip.open", MagicMock()), \
                patch("csv.reader", MagicMock(return_value=iter([[], ['foo:int', 'bar:str', 'spam:float'],
                                                                 ['123', 'test', '0.123'],
                                                                 ['234', 'another', '0.456']]))):
            csvdb = CsvDB('/foobar')
            csvdb.open()

            entities = csvdb.get(FoobarEntity, eq={'foo': 123})
            assert list == type(entities)
            assert len(entities) == 1

            assert entities[0].foo == 123
            assert entities[0].bar == 'test'
            assert entities[0].spam == 0.123

    def test_get_obj_more_than(self):
        '''
        Getting an object from the store with conditions
        :return:
        '''
        with patch("gzip.open", MagicMock()), \
                patch("os.listdir", MagicMock(return_value=['test_db'])), \
                patch("csv.reader", MagicMock(return_value=iter([[], ['foo:int', 'bar:str', 'spam:float'],
                                                                 ['123', 'test', '0.123'],
                                                                 ['234', 'another', '0.456']]))):
            csvdb = CsvDB('/foobar')
            csvdb.open()

            entities = csvdb.get(FoobarEntity, mt={'foo': 123})
            assert list == type(entities)
            assert len(entities) == 1

            assert entities[0].foo == 234
            assert entities[0].bar == 'another'
            assert entities[0].spam == 0.456

    def test_get_obj_less_than(self):
        '''
        Getting an object from the store with conditions
        :return:
        '''
        with patch("gzip.open", MagicMock()), \
                patch("os.listdir", MagicMock(return_value=['test_db'])), \
                patch("csv.reader", MagicMock(return_value=iter([[], ['foo:int', 'bar:str', 'spam:float'],
                                                                 ['123', 'test', '0.123'],
                                                                 ['234', 'another', '0.456']]))):
            csvdb = CsvDB('/foobar')
            csvdb.open()

            entities = csvdb.get(FoobarEntity, lt={'foo': 234})
            assert list == type(entities)
            assert len(entities) == 1

            assert entities[0].foo == 123
            assert entities[0].bar == 'test'
            assert entities[0].spam == 0.123

    def test_get_obj_matching(self):
        '''
        Getting an object from the store with conditions
        :return:
        '''
        with patch("gzip.open", MagicMock()), \
                patch("os.listdir", MagicMock(return_value=['test_db'])), \
                patch("csv.reader", MagicMock(return_value=iter([[], ['foo:int', 'bar:str', 'spam:float'],
                                                                 ['123', 'this is test of something', '0.123'],
                                                                 ['234', 'another test of stuff', '0.456']]))):
            csvdb = CsvDB('/foobar')
            csvdb.open()

            entities = csvdb.get(FoobarEntity, matches={'bar': r'is\stest'})
            assert list == type(entities)
            assert len(entities) == 1

            assert entities[0].foo == 123
            assert entities[0].bar == 'this is test of something'
            assert entities[0].spam == 0.123

    def test_obj_serialization(self):
        '''
        Test object serialization.
        :return:
        '''
        obj = FoobarEntity()
        obj.foo = 123
        obj.bar = 'test entity'
        obj.spam = 0.123

        descr = OrderedDict([tuple(elm.split(':')) for elm in ["foo:int", "bar:str", "spam:float"]])
        assert obj._serialize(descr) == [123, 'test entity', 0.123]

    def test_obj_validation(self):
        '''
        Test object validation.

        :return:
        '''
        with patch("os.path.exists", MagicMock(return_value=False)), \
                patch("os.listdir", MagicMock(return_value=['some_table'])):
            obj = FoobarEntity()
            obj.foo = 123
            obj.bar = 'test entity'
            obj.spam = 0.123

            csvdb = CsvDB('/foobar')
            csvdb._tables = {'some_table': OrderedDict([tuple(elm.split(':'))
                                                        for elm in ["foo:int", "bar:str", "spam:float"]])}
            assert csvdb._validate_object(obj) == [123, 'test entity', 0.123]

    def test_criteria(self):
        '''
        Test criteria selector.

        :return:
        '''
        with patch("os.path.exists", MagicMock(return_value=False)), \
                patch("os.listdir", MagicMock(return_value=['some_table'])):
            obj = FoobarEntity()
            obj.foo = 123
            obj.bar = 'test entity'
            obj.spam = 0.123
            obj.pi = 3.14

            cmp = CsvDB('/foobar')._CsvDB__criteria

            # Single
            assert cmp(obj, eq={'foo': 123}) is True
            assert cmp(obj, lt={'foo': 124}) is True
            assert cmp(obj, mt={'foo': 122}) is True

            assert cmp(obj, eq={'foo': 0}) is False
            assert cmp(obj, lt={'foo': 123}) is False
            assert cmp(obj, mt={'foo': 123}) is False

            assert cmp(obj, matches={'bar': r't\se.*?'}) is True
            assert cmp(obj, matches={'bar': r'\s\sentity'}) is False

            # Combined
            assert cmp(obj, eq={'foo': 123, 'bar': r'test entity', 'spam': 0.123}) is True
            assert cmp(obj, eq={'foo': 123, 'bar': r'test', 'spam': 0.123}) is False

            assert cmp(obj, lt={'foo': 124, 'spam': 0.124}) is True
            assert cmp(obj, lt={'foo': 124, 'spam': 0.123}) is False

            assert cmp(obj, mt={'foo': 122, 'spam': 0.122}) is True
            assert cmp(obj, mt={'foo': 122, 'spam': 0.123}) is False

            assert cmp(obj, matches={'bar': r'test'}, mt={'foo': 122}, lt={'spam': 0.124}, eq={'pi': 3.14}) is True

            assert cmp(obj, matches={'bar': r'^test.*?y$'}, mt={'foo': 122}, lt={'spam': 0.124}, eq={'pi': 3.14}) is True
            assert cmp(obj, matches={'bar': r'^ent'}, mt={'foo': 122}, lt={'spam': 0.124}, eq={'pi': 3.14}) is False
            assert cmp(obj, matches={'bar': r'^test.*?y$'}, mt={'foo': 123}, lt={'spam': 0.124}, eq={'pi': 3.14}) is False



# -*- coding: utf-8 -*-
'''
Unit tests for the docker state
'''

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    MagicMock,
    NO_MOCK,
    NO_MOCK_REASON,
    patch
)

# Import Salt Libs
import salt.modules.dockermod as docker_mod
import salt.states.docker_image as docker_state


@skipIf(NO_MOCK, NO_MOCK_REASON)
class DockerImageTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test docker_image states
    '''
    def setup_loader_modules(self):
        return {
            docker_mod: {
                '__context__': {'docker.docker_version': ''}
            },
            docker_state: {
                '__opts__': {'test': False}
            }
        }

    def test_present_already_local(self):
        '''
        According following sls,

        .. code-block:: yaml

            image:latest:
              docker_image.present:
                - force: true

        if ``image:latest`` is already downloaded locally the state
        should not report changes.
        '''
        docker_inspect_image = MagicMock(return_value={'Id': 'abcdefghijkl'})
        docker_pull = MagicMock(
            return_value={'Layers':
                          {'Already_Pulled': ['abcdefghijkl'],
                           'Pulled': []},
                          'Status': 'Image is up to date for image:latest',
                          'Time_Elapsed': 1.1})
        docker_list_tags = MagicMock(return_value=['image:latest'])
        docker_resolve_tag = MagicMock(return_value='image:latest')
        __salt__ = {'docker.list_tags': docker_list_tags,
                    'docker.pull': docker_pull,
                    'docker.inspect_image': docker_inspect_image,
                    'docker.resolve_tag': docker_resolve_tag}
        with patch.dict(docker_state.__dict__,
                        {'__salt__': __salt__}):
            ret = docker_state.present('image:latest', force=True)
            self.assertEqual(ret,
                             {'changes': {},
                              'result': True,
                              'comment': "Image 'image:latest' was pulled, "
                              "but there were no changes",
                              'name': 'image:latest',
                              })

    def test_present_and_force(self):
        '''
        According following sls,

        .. code-block:: yaml

            image:latest:
              docker_image.present:
                - force: true

        if ``image:latest`` is not downloaded and force is true
        should pull a new image successfully.
        '''
        docker_inspect_image = MagicMock(return_value={'Id': '1234567890ab'})
        docker_pull = MagicMock(
            return_value={'Layers':
                          {'Pulled': ['abcdefghijkl']},
                           'Status': "Image 'image:latest' was pulled",
                           'Time_Elapsed': 1.1})
        docker_list_tags = MagicMock(side_effect=[[], ['image:latest']])
        docker_resolve_tag = MagicMock(return_value='image:latest')
        __salt__ = {'docker.list_tags': docker_list_tags,
                    'docker.pull': docker_pull,
                    'docker.inspect_image': docker_inspect_image,
                    'docker.resolve_tag': docker_resolve_tag}
        with patch.dict(docker_state.__dict__,
                        {'__salt__': __salt__}):
            ret = docker_state.present('image:latest', force=True)
        self.assertEqual(ret,
                         {'changes': {
                             'Layers': {'Pulled': ['abcdefghijkl']},
                             'Status': "Image 'image:latest' was pulled",
                             'Time_Elapsed': 1.1},
                             'result': True,
                             'comment': "Image 'image:latest' was pulled",
                             'name': 'image:latest',
                         })



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import os
import textwrap

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.unit import skipIf
from tests.support.helpers import destructiveTest
from tests.support.runtests import RUNTIME_VARS

# Import Salt libs
import salt.utils.files
import salt.utils.platform

REPO_DIR = os.path.join(RUNTIME_VARS.FILES, 'file', 'base', 'win', 'repo-ng')
CURL = os.path.join(REPO_DIR, 'curl.sls')


@skipIf(not salt.utils.platform.is_windows(), 'windows test only')
class WinPKGTest(ModuleCase):
    '''
    Tests for salt.modules.win_pkg. There are already
    some pkg execution module tests in the the test
    integration.modules.test_pkg but this will be for
    specific windows software respository tests while
    using the win_pkg module.
    '''
    @destructiveTest
    def test_adding_removing_pkg_sls(self):
        '''
        Test add and removing a new pkg sls
        in the windows software repository
        '''
        def _check_pkg(pkgs, check_refresh, exists=True):
            refresh = self.run_function('pkg.refresh_db')
            self.assertEqual(check_refresh, refresh['total'],
                 msg='total returned {0}. Expected return {1}'.format(refresh['total'], check_refresh))
            repo_data = self.run_function('pkg.get_repo_data')
            repo_cache = os.path.join(RUNTIME_VARS.TMP, 'rootdir', 'cache', 'files', 'base', 'win', 'repo-ng')
            for pkg in pkgs:
                if exists:
                    assert pkg in str(repo_data), str(repo_data)
                else:
                    assert pkg not in str(repo_data), str(repo_data)

                for root, dirs, files in os.walk(repo_cache):
                    if exists:
                        assert pkg + '.sls' in files
                    else:
                        assert pkg + '.sls' not in files

        pkgs = ['putty', '7zip']
        # check putty and 7zip are in cache and repo query
        _check_pkg(pkgs, 2)

        # now add new sls
        with salt.utils.files.fopen(CURL, 'w') as fp_:
            fp_.write(textwrap.dedent('''
                curl:
                  '7.46.0':
                    full_name: 'cURL'
                    {% if grains['cpuarch'] == 'AMD64' %}
                    installer: 'salt://win/repo-ng/curl/curl-7.46.0-win64.msi'
                    uninstaller: 'salt://win/repo-ng/curl/curl-7.46.0-win64.msi'
                    {% else %}
                    installer: 'salt://win/repo-ng/curl/curl-7.46.0-win32.msi'
                    uninstaller: 'salt://win/repo-ng/curl/curl-7.46.0-win32.msi'
                    {% endif %}
                    install_flags: '/qn /norestart'
                    uninstall_flags: '/qn /norestart'
                    msiexec: True
                    locale: en_US
                    reboot: False
                '''))
        # now check if curl is also in cache and repo query
        pkgs.append('curl')
        for pkg in pkgs:
            self.assertIn(pkg + '.sls', os.listdir(REPO_DIR))
        _check_pkg(pkgs, 3)

        # remove curl sls and check its not in cache and repo query
        os.remove(CURL)
        _check_pkg(['curl'], 2, exists=False)

    def tearDown(self):
        if os.path.isfile(CURL):
            os.remove(CURL)



# -*- coding: utf-8 -*-
'''
test for pillar file_tree.py
'''

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

import os
import tempfile
import shutil

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, patch, MagicMock
from tests.support.paths import TMP
from tests.support.helpers import TestsLoggingHandler

# Import Salt Libs
import salt.utils.files
import salt.utils.stringutils
import salt.pillar.file_tree as file_tree


MINION_ID = 'test-host'
NODEGROUP_PATH = os.path.join('nodegroups', 'test-group', 'files')
HOST_PATH = os.path.join('hosts', MINION_ID, 'files')

BASE_PILLAR_CONTENT = {'files': {'hostfile': b'base', 'groupfile': b'base'}}
DEV_PILLAR_CONTENT = {'files': {'hostfile': b'base', 'groupfile': b'dev2',
                                'hostfile1': b'dev1', 'groupfile1': b'dev1',
                                'hostfile2': b'dev2'}}
PARENT_PILLAR_CONTENT = {'files': {'hostfile': b'base', 'groupfile': b'base',
                                   'hostfile2': b'dev2'}}

FILE_DATA = {
    os.path.join('base', HOST_PATH, 'hostfile'): 'base',
    os.path.join('dev1', HOST_PATH, 'hostfile1'): 'dev1',
    os.path.join('dev2', HOST_PATH, 'hostfile2'): 'dev2',
    os.path.join('base', NODEGROUP_PATH, 'groupfile'): 'base',
    os.path.join('dev1', NODEGROUP_PATH, 'groupfile1'): 'dev1',
    os.path.join('dev2', NODEGROUP_PATH, 'groupfile'): 'dev2'  # test merging
}

_CHECK_MINIONS_RETURN = {'minions': [MINION_ID], 'missing': []}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class FileTreePillarTestCase(TestCase, LoaderModuleMockMixin):
    'test file_tree pillar'
    maxDiff = None

    def setup_loader_modules(self):
        self.tmpdir = tempfile.mkdtemp(dir=TMP)
        self.addCleanup(shutil.rmtree, self.tmpdir)
        cachedir = os.path.join(self.tmpdir, 'cachedir')
        os.makedirs(os.path.join(cachedir, 'file_tree'))
        self.pillar_path = self._create_pillar_files()
        return {
            file_tree: {
                '__opts__':  {
                    'cachedir': cachedir,
                    'pillar_roots': {
                        'base':   [os.path.join(self.pillar_path, 'base')],
                        'dev':    [os.path.join(self.pillar_path, 'base'),
                                   os.path.join(self.pillar_path, 'dev1'),
                                   os.path.join(self.pillar_path, 'dev2')
                                   ],
                        'parent': [os.path.join(self.pillar_path, 'base', 'sub1'),
                                   os.path.join(self.pillar_path, 'dev2', 'sub'),
                                   os.path.join(self.pillar_path, 'base', 'sub2'),
                                   ],
                    },
                    'pillarenv': 'base',
                    'nodegroups': {'test-group': [MINION_ID]},
                    'optimization_order': [0, 1, 2],
                    'file_buffer_size': 262144,
                    'file_roots': {'base': '', 'dev': '', 'parent': ''},
                    'extension_modules': '',
                    'renderer': 'yaml_jinja',
                    'renderer_blacklist': [],
                    'renderer_whitelist': []
                }
            }
        }

    def _create_pillar_files(self):
        'create files in tempdir'
        pillar_path = os.path.join(self.tmpdir, 'file_tree')
        for filename in FILE_DATA:
            filepath = os.path.join(pillar_path, filename)
            os.makedirs(os.path.dirname(filepath))
            with salt.utils.files.fopen(filepath, 'w') as data_file:
                data_file.write(salt.utils.stringutils.to_str(FILE_DATA[filename]))
        return pillar_path

    def test_absolute_path(self):
        'check file tree is imported correctly with an absolute path'
        absolute_path = os.path.join(self.pillar_path, 'base')
        with patch('salt.utils.minions.CkMinions.check_minions', MagicMock(return_value=_CHECK_MINIONS_RETURN)):
            mypillar = file_tree.ext_pillar(MINION_ID, None, absolute_path)
            self.assertEqual(BASE_PILLAR_CONTENT, mypillar)

            with patch.dict(file_tree.__opts__, {'pillarenv': 'dev'}):
                mypillar = file_tree.ext_pillar(MINION_ID, None, absolute_path)
                self.assertEqual(BASE_PILLAR_CONTENT, mypillar)

    def test_relative_path(self):
        'check file tree is imported correctly with a relative path'
        with patch('salt.utils.minions.CkMinions.check_minions', MagicMock(return_value=_CHECK_MINIONS_RETURN)):
            mypillar = file_tree.ext_pillar(MINION_ID, None, '.')
            self.assertEqual(BASE_PILLAR_CONTENT, mypillar)

            with patch.dict(file_tree.__opts__, {'pillarenv': 'dev'}):
                mypillar = file_tree.ext_pillar(MINION_ID, None, '.')
                self.assertEqual(DEV_PILLAR_CONTENT, mypillar)

    def test_parent_path(self):
        'check if file tree is merged correctly with a .. path'
        with patch('salt.utils.minions.CkMinions.check_minions', MagicMock(return_value=_CHECK_MINIONS_RETURN)):
            with patch.dict(file_tree.__opts__, {'pillarenv': 'parent'}):
                mypillar = file_tree.ext_pillar(MINION_ID, None, '..')
                self.assertEqual(PARENT_PILLAR_CONTENT, mypillar)

    def test_no_pillarenv(self):
        'confirm that file_tree yells when pillarenv is missing for a relative path'
        with patch('salt.utils.minions.CkMinions.check_minions', MagicMock(return_value=_CHECK_MINIONS_RETURN)):
            with patch.dict(file_tree.__opts__, {'pillarenv': None}):
                with TestsLoggingHandler() as handler:
                    mypillar = file_tree.ext_pillar(MINION_ID, None, '.')
                    self.assertEqual({}, mypillar)

                    for message in handler.messages:
                        if message.startswith('ERROR:') and 'pillarenv is not set' in message:
                            break
                    else:
                        raise AssertionError('Did not find error message')
